@@ -54,8 +54,7 @@ public:
     QString value() const { return typedValue<QString>(valueKey); }
     void setValue(const QString &value) { insert(valueKey, value); }
 
-    bool isValid(QStringList *error) const override
-    { return check<QString>(error, languageKey) && check<QString>(error, valueKey); }
+    bool isValid() const override { return contains(languageKey) && contains(valueKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT MarkedString
@@ -71,6 +70,7 @@ public:
     {}
     explicit MarkedString(const QJsonValue &value);
 
+    bool isValid() const;
     operator QJsonValue() const;
 };
 
@@ -83,7 +83,7 @@ public:
     explicit HoverContent(const QList<MarkedString> &other) : variant(other) {}
     explicit HoverContent(const MarkupContent &other) : variant(other) {}
     explicit HoverContent(const QJsonValue &value);
-    bool isValid(QStringList *errorHierarchy) const;
+    bool isValid() const;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT Hover : public JsonObject
@@ -98,15 +98,23 @@ public:
     void setRange(const Range &range) { insert(rangeKey, range); }
     void clearRange() { remove(rangeKey); }
 
-    bool isValid(QStringList *error) const override
-    { return check<HoverContent>(error, contentsKey) && checkOptional<Range>(error, rangeKey); }
+    bool isValid() const override { return contains(contentsKey); }
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT HoverResult : public Utils::variant<Hover, std::nullptr_t>
+{
+public:
+    HoverResult() : variant(nullptr) {}
+    explicit HoverResult(const Hover &hover) : variant(hover) {}
+    explicit HoverResult(const QJsonValue &value);
+    bool isValid() const;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT HoverRequest
-    : public Request<Hover, std::nullptr_t, TextDocumentPositionParams>
+    : public Request<HoverResult, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    HoverRequest(const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit HoverRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/hover";
 };
@@ -128,11 +136,7 @@ public:
     { insert(documentationKey, documentation.toJson()); }
     void clearDocumentation() { remove(documentationKey); }
 
-    bool isValid(QStringList *error) const override
-    {
-        return check<QString>(error, labelKey)
-                && checkOptional<MarkupOrString>(error, documentationKey);
-    }
+    bool isValid() const override { return contains(labelKey); }
 };
 
 /**
@@ -150,6 +154,10 @@ public:
     void setParameters(const QList<ParameterInformation> &parameters)
     { insertArray(parametersKey, parameters); }
     void clearParameters() { remove(parametersKey); }
+
+    Utils::optional<int> activeParameter() const { return optionalValue<int>(activeParameterKey); }
+    void setActiveParameter(int activeParameter) { insert(activeParameterKey, activeParameter); }
+    void clearActiveParameter() { remove(activeParameterKey); }
 };
 
 /**
@@ -194,14 +202,14 @@ public:
     void setActiveParameter(int activeParameter) { insert(activeParameterKey, activeParameter); }
     void clearActiveParameter() { remove(activeParameterKey); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override { return contains(signaturesKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT SignatureHelpRequest
     : public Request<LanguageClientValue<SignatureHelp>, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    SignatureHelpRequest(const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit SignatureHelpRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/signatureHelp";
 };
@@ -211,7 +219,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT GotoResult
         : public Utils::variant<Location, QList<Location>, std::nullptr_t>
 {
 public:
-    GotoResult(const QJsonValue &value);
+    explicit GotoResult(const QJsonValue &value);
     using variant::variant;
 };
 
@@ -219,7 +227,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT GotoDefinitionRequest
     : public Request<GotoResult, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    GotoDefinitionRequest(const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit GotoDefinitionRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/definition";
 };
@@ -228,8 +236,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT GotoTypeDefinitionRequest : public Request<
         GotoResult, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    GotoTypeDefinitionRequest(
-            const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit GotoTypeDefinitionRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/typeDefinition";
 };
@@ -238,8 +245,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT GotoImplementationRequest : public Request<
         GotoResult, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    GotoImplementationRequest(
-            const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit GotoImplementationRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/implementation";
 };
@@ -260,24 +266,21 @@ public:
         void setIncludeDeclaration(bool includeDeclaration)
         { insert(includeDeclarationKey, includeDeclaration); }
 
-        bool isValid(QStringList *error) const override
-        { return check<bool>(error, includeDeclarationKey); }
+        bool isValid() const override { return contains(includeDeclarationKey); }
     };
 
     ReferenceContext context() const { return typedValue<ReferenceContext>(contextKey); }
     void setContext(const ReferenceContext &context) { insert(contextKey, context); }
 
-    bool isValid(QStringList *error) const override
-    {
-        return TextDocumentPositionParams::isValid(error)
-                && check<ReferenceContext>(error, contextKey); }
+    bool isValid() const override
+    { return TextDocumentPositionParams::isValid() && contains(contextKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT FindReferencesRequest : public Request<
         LanguageClientArray<Location>, std::nullptr_t, ReferenceParams>
 {
 public:
-    FindReferencesRequest(const ReferenceParams &params = ReferenceParams());
+    explicit FindReferencesRequest(const ReferenceParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/references";
 };
@@ -300,8 +303,7 @@ public:
     void setKind(int kind) { insert(kindKey, kind); }
     void clearKind() { remove(kindKey); }
 
-    bool isValid(QStringList *error) const override
-    { return check<Range>(error, rangeKey) && checkOptional<int>(error, kindKey); }
+    bool isValid() const override { return contains(rangeKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentHighlightsResult
@@ -310,7 +312,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentHighlightsResult
 public:
     using variant::variant;
     DocumentHighlightsResult() : variant(nullptr) {}
-    DocumentHighlightsResult(const QJsonValue &value);
+    explicit DocumentHighlightsResult(const QJsonValue &value);
     using variant::operator=;
 };
 
@@ -318,8 +320,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentHighlightsRequest : public Request<
         DocumentHighlightsResult, std::nullptr_t, TextDocumentPositionParams>
 {
 public:
-    DocumentHighlightsRequest(
-            const TextDocumentPositionParams &params = TextDocumentPositionParams());
+    explicit DocumentHighlightsRequest(const TextDocumentPositionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/documentHighlight";
 };
@@ -328,7 +329,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT TextDocumentParams : public JsonObject
 {
 public:
     TextDocumentParams();
-    TextDocumentParams(const TextDocumentIdentifier &identifier);
+    explicit TextDocumentParams(const TextDocumentIdentifier &identifier);
     using JsonObject::JsonObject;
 
     TextDocumentIdentifier textDocument() const
@@ -336,8 +337,7 @@ public:
     void setTextDocument(const TextDocumentIdentifier &textDocument)
     { insert(textDocumentKey, textDocument); }
 
-    bool isValid(QStringList *error) const override
-    { return check<TextDocumentIdentifier>(error, textDocumentKey); }
+    bool isValid() const override { return contains(textDocumentKey); }
 };
 
 using DocumentSymbolParams = TextDocumentParams;
@@ -348,7 +348,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentSymbolsResult
 public:
     using variant::variant;
     DocumentSymbolsResult() : variant(nullptr) {}
-    DocumentSymbolsResult(const QJsonValue &value);
+    explicit DocumentSymbolsResult(const QJsonValue &value);
     DocumentSymbolsResult(const DocumentSymbolsResult &other) : variant(other) {}
     DocumentSymbolsResult(DocumentSymbolsResult &&other) : variant(std::move(other)) {}
 
@@ -363,6 +363,8 @@ public:
         variant::operator=(other);
         return *this;
     }
+    // Needed to make it usable in Qt 6 signals.
+    bool operator<(const DocumentSymbolsResult &other) const = delete;
 };
 
 
@@ -370,7 +372,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentSymbolsRequest
         : public Request<DocumentSymbolsResult, std::nullptr_t, DocumentSymbolParams>
 {
 public:
-    DocumentSymbolsRequest(const DocumentSymbolParams &params = DocumentSymbolParams());
+    explicit DocumentSymbolsRequest(const DocumentSymbolParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/documentSymbol";
 };
@@ -418,7 +420,7 @@ public:
         void setOnly(const QList<CodeActionKind> &only);
         void clearOnly() { remove(onlyKey); }
 
-        bool isValid(QStringList *error) const override;
+        bool isValid() const override { return contains(diagnosticsKey); }
     };
 
     TextDocumentIdentifier textDocument() const
@@ -432,7 +434,8 @@ public:
     CodeActionContext context() const { return typedValue<CodeActionContext>(contextKey); }
     void setContext(const CodeActionContext &context) { insert(contextKey, context); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override
+    { return contains(textDocumentKey) && contains(rangeKey) && contains(contextKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT CodeAction : public JsonObject
@@ -461,7 +464,7 @@ public:
     void setCommand(const Command &command) { insert(commandKey, command); }
     void clearCommand() { remove(commandKey); }
 
-    bool isValid(QStringList *) const override;
+    bool isValid() const override { return contains(titleKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT CodeActionResult
@@ -476,7 +479,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT CodeActionRequest : public Request<
         CodeActionResult, std::nullptr_t, CodeActionParams>
 {
 public:
-    CodeActionRequest(const CodeActionParams &params = CodeActionParams());
+    explicit CodeActionRequest(const CodeActionParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/codeAction";
 };
@@ -495,19 +498,18 @@ public:
     void setCommand(const Command &command) { insert(commandKey, command); }
     void clearCommand() { remove(commandKey); }
 
-    Utils::optional<QJsonValue> data() const { return optionalValue<QJsonValue>(dataKey); }
+    Utils::optional<QJsonValue> data() const;
     void setData(const QJsonValue &data) { insert(dataKey, data); }
     void clearData() { remove(dataKey); }
 
-    bool isValid(QStringList *error) const override
-    { return check<Range>(error, rangeKey) && checkOptional<Command>(error, commandKey); }
+    bool isValid() const override { return contains(rangeKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT CodeLensRequest : public Request<
         LanguageClientArray<CodeLens>, std::nullptr_t, CodeLensParams>
 {
 public:
-    CodeLensRequest(const CodeLensParams &params = CodeLensParams());
+    explicit CodeLensRequest(const CodeLensParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/codeLens";
 };
@@ -516,7 +518,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT CodeLensResolveRequest : public Request<
         CodeLens, std::nullptr_t, CodeLens>
 {
 public:
-    CodeLensResolveRequest(const CodeLens &params = CodeLens());
+    explicit CodeLensResolveRequest(const CodeLens &params);
     using Request::Request;
     constexpr static const char methodName[] = "codeLens/resolve";
 };
@@ -533,12 +535,11 @@ public:
     void setTarget(const DocumentUri &target) { insert(targetKey, target.toString()); }
     void clearTarget() { remove(targetKey); }
 
-    Utils::optional<QJsonValue> data() const { return optionalValue<QJsonValue>(dataKey); }
+    Utils::optional<QJsonValue> data() const;
     void setData(const QJsonValue &data) { insert(dataKey, data); }
     void clearData() { remove(dataKey); }
 
-    bool isValid(QStringList *error) const override
-    { return check<Range>(error, rangeKey) && checkOptional<QString>(error, targetKey); }
+    bool isValid() const override { return contains(rangeKey); }
 };
 
 using DocumentLinkParams = TextDocumentParams;
@@ -547,7 +548,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentLinkRequest : public Request<
         LanguageClientValue<DocumentLink>, std::nullptr_t, DocumentLinkParams>
 {
 public:
-    DocumentLinkRequest(const DocumentLinkParams &params = DocumentLinkParams());
+    explicit DocumentLinkRequest(const DocumentLinkParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/documentLink";
 };
@@ -556,7 +557,7 @@ class LANGUAGESERVERPROTOCOL_EXPORT DocumentLinkResolveRequest : public Request<
         DocumentLink, std::nullptr_t, DocumentLink>
 {
 public:
-    DocumentLinkResolveRequest(const DocumentLink &params = DocumentLink());
+    explicit DocumentLinkResolveRequest(const DocumentLink &params);
     using Request::Request;
     constexpr static const char methodName[] = "documentLink/resolve";
 };
@@ -580,7 +581,8 @@ public:
     double alpha() const { return typedValue<double>(alphaKey); }
     void setAlpha(double alpha) { insert(alphaKey, alpha); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override
+    { return contains(redKey) && contains(greenKey) && contains(blueKey) && contains(alphaKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ColorInformation : public JsonObject
@@ -594,15 +596,14 @@ public:
     Color color() const { return typedValue<Color>(colorKey); }
     void setColor(const Color &color) { insert(colorKey, color); }
 
-    bool isValid(QStringList *error) const override
-    { return check<Range>(error, rangeKey) && check<Color>(error, colorKey); }
+    bool isValid() const override { return contains(rangeKey) && contains(colorKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentColorRequest : public Request<
         QList<ColorInformation>, std::nullptr_t, DocumentColorParams>
 {
 public:
-    DocumentColorRequest(const DocumentColorParams &params = DocumentColorParams());
+    explicit DocumentColorRequest(const DocumentColorParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/documentColor";
 };
@@ -623,7 +624,8 @@ public:
     Range range() const { return typedValue<Range>(rangeKey); }
     void setRange(const Range &range) { insert(rangeKey, range); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override
+    { return contains(textDocumentKey) && contains(colorInfoKey) && contains(rangeKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ColorPresentation : public JsonObject
@@ -644,19 +646,14 @@ public:
     { insertArray(additionalTextEditsKey, additionalTextEdits); }
     void clearAdditionalTextEdits() { remove(additionalTextEditsKey); }
 
-    bool isValid(QStringList *error) const override
-    {
-        return check<QString>(error, labelKey)
-                && checkOptional<TextEdit>(error, textEditKey)
-                && checkOptionalArray<TextEdit>(error, additionalTextEditsKey);
-    }
+    bool isValid() const override { return contains(labelKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT ColorPresentationRequest : public Request<
         QList<ColorPresentation>, std::nullptr_t, ColorPresentationParams>
 {
 public:
-    ColorPresentationRequest(const ColorPresentationParams &params = ColorPresentationParams());
+    explicit ColorPresentationRequest(const ColorPresentationParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/colorPresentation";
 };
@@ -665,14 +662,12 @@ class DocumentFormattingProperty : public Utils::variant<bool, double, QString>
 {
 public:
     DocumentFormattingProperty() = default;
-    DocumentFormattingProperty(const QJsonValue &value);
-    DocumentFormattingProperty(const DocumentFormattingProperty &other)
+    explicit DocumentFormattingProperty(const QJsonValue &value);
+    explicit DocumentFormattingProperty(const DocumentFormattingProperty &other)
         : Utils::variant<bool, double, QString>(other) {}
 
     using variant::variant;
     using variant::operator=;
-
-    bool isValid(QStringList *error) const;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT FormattingOptions : public JsonObject
@@ -686,11 +681,29 @@ public:
     bool insertSpace() const { return typedValue<bool>(insertSpaceKey); }
     void setInsertSpace(bool insertSpace) { insert(insertSpaceKey, insertSpace); }
 
+    Utils::optional<bool> trimTrailingWhitespace() const
+    { return optionalValue<bool>(trimTrailingWhitespaceKey); }
+    void setTrimTrailingWhitespace(bool trimTrailingWhitespace)
+    { insert(trimTrailingWhitespaceKey, trimTrailingWhitespace); }
+    void clearTrimTrailingWhitespace() { remove(trimTrailingWhitespaceKey); }
+
+    Utils::optional<bool> insertFinalNewline() const
+    { return optionalValue<bool>(insertFinalNewlineKey); }
+    void setInsertFinalNewline(bool insertFinalNewline)
+    { insert(insertFinalNewlineKey, insertFinalNewline); }
+    void clearInsertFinalNewline() { remove(insertFinalNewlineKey); }
+
+    Utils::optional<bool> trimFinalNewlines() const
+    { return optionalValue<bool>(trimFinalNewlinesKey); }
+    void setTrimFinalNewlines(bool trimFinalNewlines)
+    { insert(trimFinalNewlinesKey, trimFinalNewlines); }
+    void clearTrimFinalNewlines() { remove(trimFinalNewlinesKey); }
+
     QHash<QString, DocumentFormattingProperty> properties() const;
     void setProperty(const QString &key, const DocumentFormattingProperty &property);
     void removeProperty(const QString &key) { remove(key); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override { return contains(insertSpaceKey) && contains(tabSizeKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentFormattingParams : public JsonObject
@@ -706,14 +719,14 @@ public:
     FormattingOptions options() const { return typedValue<FormattingOptions>(optionsKey); }
     void setOptions(const FormattingOptions &options) { insert(optionsKey, options); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override { return contains(textDocumentKey) && contains(optionsKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentFormattingRequest : public Request<
         LanguageClientArray<TextEdit>, std::nullptr_t, DocumentFormattingParams>
 {
 public:
-    DocumentFormattingRequest(const DocumentFormattingParams &params = DocumentFormattingParams());
+    explicit DocumentFormattingRequest(const DocumentFormattingParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/formatting";
 };
@@ -734,14 +747,15 @@ public:
     FormattingOptions options() const { return typedValue<FormattingOptions>(optionsKey); }
     void setOptions(const FormattingOptions &options) { insert(optionsKey, options); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override
+    { return contains(textDocumentKey) && contains(rangeKey) && contains(optionsKey); }
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentRangeFormattingRequest : public Request<
-        QList<TextEdit>, std::nullptr_t, DocumentFormattingParams>
+        LanguageClientArray<TextEdit>, std::nullptr_t, DocumentRangeFormattingParams>
 {
 public:
-    DocumentRangeFormattingRequest(const DocumentFormattingParams &params = DocumentFormattingParams());
+    explicit DocumentRangeFormattingRequest(const DocumentRangeFormattingParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/rangeFormatting";
 };
@@ -765,19 +779,54 @@ public:
     FormattingOptions options() const { return typedValue<FormattingOptions>(optionsKey); }
     void setOptions(const FormattingOptions &options) { insert(optionsKey, options); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT DocumentOnTypeFormattingRequest : public Request<
-        QList<TextEdit>, std::nullptr_t, DocumentFormattingParams>
+        LanguageClientArray<TextEdit>, std::nullptr_t, DocumentOnTypeFormattingParams>
 {
 public:
-    DocumentOnTypeFormattingRequest(
-            const DocumentFormattingParams &params = DocumentFormattingParams());
+    explicit DocumentOnTypeFormattingRequest(const DocumentOnTypeFormattingParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/onTypeFormatting";
 };
 
+class PlaceHolderResult : public JsonObject
+{
+public:
+    using JsonObject::JsonObject;
+
+    Range range() const { return typedValue<Range>(rangeKey); }
+    void setRange(const Range &range) { insert(rangeKey, range); }
+
+    QString placeHolder() const { return typedValue<QString>(placeHolderKey); }
+    void setPlaceHolder(const QString &placeHolder) { insert(placeHolderKey, placeHolder); }
+
+    bool isValid() const override { return contains(rangeKey); }
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT PrepareRenameResult
+    : public Utils::variant<PlaceHolderResult, Range, std::nullptr_t>
+{
+public:
+    PrepareRenameResult();
+    PrepareRenameResult(const Utils::variant<PlaceHolderResult, Range, std::nullptr_t> &val);
+    explicit PrepareRenameResult(const PlaceHolderResult &val);
+    explicit PrepareRenameResult(const Range &val);
+    explicit PrepareRenameResult(const QJsonValue &val);
+
+    bool isValid() const;
+};
+
+class LANGUAGESERVERPROTOCOL_EXPORT PrepareRenameRequest
+    : public Request<PrepareRenameResult, std::nullptr_t, TextDocumentPositionParams>
+{
+public:
+    explicit PrepareRenameRequest(const TextDocumentPositionParams &params);
+    using Request::Request;
+    constexpr static const char methodName[] = "textDocument/prepareRename";
+};
+
 class LANGUAGESERVERPROTOCOL_EXPORT RenameParams : public JsonObject
 {
 public:
@@ -794,14 +843,14 @@ public:
     QString newName() const { return typedValue<QString>(newNameKey); }
     void setNewName(const QString &newName) { insert(newNameKey, newName); }
 
-    bool isValid(QStringList *error) const override;
+    bool isValid() const override;
 };
 
 class LANGUAGESERVERPROTOCOL_EXPORT RenameRequest : public Request<
         WorkspaceEdit, std::nullptr_t, RenameParams>
 {
 public:
-    RenameRequest(const RenameParams &params = RenameParams());
+    explicit RenameRequest(const RenameParams &params);
     using Request::Request;
     constexpr static const char methodName[] = "textDocument/rename";
 };
