@@ -35,6 +35,8 @@
 #include <QFile>
 #include <QIODevice>
 
+using namespace Utils;
+
 namespace CMakeProjectManager {
 
 // --------------------------------------------------------------------
@@ -49,23 +51,36 @@ CMakeConfigItem::CMakeConfigItem(const QByteArray &k, Type t,
     key(k), type(t), value(v), documentation(d), values(s)
 { }
 
+CMakeConfigItem::CMakeConfigItem(const QByteArray &k, Type t, const QByteArray &v) :
+    key(k), type(t), value(v)
+{ }
+
 CMakeConfigItem::CMakeConfigItem(const QByteArray &k, const QByteArray &v) :
     key(k), value(v)
 { }
 
-QByteArray CMakeConfigItem::valueOf(const QByteArray &key, const QList<CMakeConfigItem> &input)
+QByteArray CMakeConfig::valueOf(const QByteArray &key) const
 {
-    for (auto it = input.constBegin(); it != input.constEnd(); ++it) {
+    for (auto it = constBegin(); it != constEnd(); ++it) {
         if (it->key == key)
             return it->value;
     }
     return QByteArray();
 }
 
-QString CMakeConfigItem::expandedValueOf(const ProjectExplorer::Kit *k, const QByteArray &key,
-                                         const QList<CMakeConfigItem> &input)
+QString CMakeConfig::stringValueOf(const QByteArray &key) const
 {
-    for (auto it = input.constBegin(); it != input.constEnd(); ++it) {
+    return QString::fromUtf8(valueOf(key));
+}
+
+FilePath CMakeConfig::filePathValueOf(const QByteArray &key) const
+{
+    return FilePath::fromUtf8(valueOf(key));
+}
+
+QString CMakeConfig::expandedValueOf(const ProjectExplorer::Kit *k, const QByteArray &key) const
+{
+    for (auto it = constBegin(); it != constEnd(); ++it) {
         if (it->key == key)
             return it->expandedValue(k);
     }
@@ -148,7 +163,6 @@ CMakeConfigItem::Type CMakeConfigItem::typeStringToType(const QByteArray &type)
     if (type == "INTERNAL")
         return CMakeConfigItem::INTERNAL;
 
-    QTC_CHECK(type == "UNINITIALIZED");
     return CMakeConfigItem::UNINITIALIZED;
 }
 
@@ -174,11 +188,11 @@ QString CMakeConfigItem::typeToTypeString(const CMakeConfigItem::Type t)
     return {};
 }
 
-Utils::optional<bool> CMakeConfigItem::toBool(const QByteArray &value)
+Utils::optional<bool> CMakeConfigItem::toBool(const QString &value)
 {
     // Taken from CMakes if(<constant>) documentation:
     // "Named boolean constants are case-insensitive."
-    const QString v = QString::fromUtf8(value).toUpper();
+    const QString v = value.toUpper();
 
     bool isInt = false;
     v.toInt(&isInt);
@@ -203,9 +217,9 @@ QString CMakeConfigItem::expandedValue(const Utils::MacroExpander *expander) con
     return expander ? expander->expand(QString::fromUtf8(value)) : QString::fromUtf8(value);
 }
 
-std::function<bool (const CMakeConfigItem &a, const CMakeConfigItem &b)> CMakeConfigItem::sortOperator()
+bool CMakeConfigItem::less(const CMakeConfigItem &a, const CMakeConfigItem &b)
 {
-    return [](const CMakeConfigItem &a, const CMakeConfigItem &b) { return a.key < b.key; };
+    return a.key < b.key;
 }
 
 CMakeConfigItem CMakeConfigItem::fromString(const QString &s)
@@ -300,7 +314,7 @@ static CMakeConfigItem unsetItemFromString(const QString &input)
     return item;
 }
 
-QList<CMakeConfigItem> CMakeConfigItem::itemsFromArguments(const QStringList &list)
+CMakeConfig CMakeConfig::fromArguments(const QStringList &list, QStringList &unknownOptions)
 {
     CMakeConfig result;
     bool inSet = false;
@@ -330,13 +344,16 @@ QList<CMakeConfigItem> CMakeConfigItem::itemsFromArguments(const QStringList &li
         }
         if (i.startsWith("-D")) {
             result.append(setItemFromString(i.mid(2)));
+            continue;
         }
-        // ignore everything else as that does not define a configuration option
+
+        unknownOptions.append(i);
     }
-    return result;
+
+    return Utils::filtered(result, [](const CMakeConfigItem &item) { return !item.key.isEmpty(); });
 }
 
-QList<CMakeConfigItem> CMakeConfigItem::itemsFromFile(const Utils::FilePath &cacheFile, QString *errorMessage)
+CMakeConfig CMakeConfig::fromFile(const Utils::FilePath &cacheFile, QString *errorMessage)
 {
     CMakeConfig result;
     QFile cache(cacheFile.toString());
@@ -393,10 +410,9 @@ QList<CMakeConfigItem> CMakeConfigItem::itemsFromFile(const Utils::FilePath &cac
         }
     }
 
-    Utils::sort(result, CMakeConfigItem::sortOperator());
+    Utils::sort(result, &CMakeConfigItem::less);
 
     return result;
-
 }
 
 QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
@@ -436,6 +452,11 @@ QString CMakeConfigItem::toString(const Utils::MacroExpander *expander) const
     return QString::fromUtf8(key) + QLatin1Char(':') + typeStr + QLatin1Char('=') + expandedValue;
 }
 
+QString CMakeConfigItem::toArgument() const
+{
+    return toArgument(nullptr);
+}
+
 QString CMakeConfigItem::toArgument(const Utils::MacroExpander *expander) const
 {
     if (isUnset)
@@ -458,7 +479,12 @@ QString CMakeConfigItem::toCMakeSetLine(const Utils::MacroExpander *expander) co
 bool CMakeConfigItem::operator==(const CMakeConfigItem &o) const
 {
     // type, isAdvanced and documentation do not matter for a match!
-    return o.key == key && o.value == value && o.isUnset == isUnset;
+    return o.key == key && o.value == value && o.isUnset == isUnset && o.isInitial == isInitial;
+}
+
+Utils::QHashValueType qHash(const CMakeConfigItem &it)
+{
+    return ::qHash(it.key) ^ ::qHash(it.value) ^ ::qHash(it.isUnset) ^ ::qHash(it.isInitial);
 }
 
 #if WITH_TESTS
