@@ -25,9 +25,9 @@
 
 #include "fileapireader.h"
 
+#include "cmakeprocess.h"
 #include "fileapidataextractor.h"
 #include "fileapiparser.h"
-#include "projecttreehelper.h"
 
 #include <coreplugin/messagemanager.h>
 
@@ -77,8 +77,7 @@ void FileApiReader::setParameters(const BuildDirParameters &p)
     qCDebug(cmakeFileApiMode) << "Work directory:" << m_parameters.buildDirectory.toUserOutput();
 
     // Reset watcher:
-    m_watcher.removeFiles(m_watcher.files());
-    m_watcher.removeDirectories(m_watcher.directories());
+    m_watcher.clear();
 
     FileApiParser::setupCMakeFileApi(m_parameters.buildDirectory, m_watcher);
 
@@ -88,14 +87,17 @@ void FileApiReader::setParameters(const BuildDirParameters &p)
 void FileApiReader::resetData()
 {
     m_cmakeFiles.clear();
-    if (!m_parameters.sourceDirectory.isEmpty())
-        m_cmakeFiles.insert(m_parameters.sourceDirectory.pathAppended("CMakeLists.txt"));
+    if (!m_parameters.sourceDirectory.isEmpty()) {
+        CMakeFileInfo cmakeListsTxt;
+        cmakeListsTxt.path = m_parameters.sourceDirectory.pathAppended("CMakeLists.txt");
+        cmakeListsTxt.isCMakeListsDotTxt = true;
+        m_cmakeFiles.insert(cmakeListsTxt);
+    }
 
     m_cache.clear();
     m_buildTargets.clear();
     m_projectParts.clear();
     m_rootProjectNode.reset();
-    m_knownHeaders.clear();
 }
 
 void FileApiReader::parse(bool forceCMakeRun,
@@ -109,8 +111,10 @@ void FileApiReader::parse(bool forceCMakeRun,
 
     const QStringList args = (forceInitialConfiguration ? m_parameters.initialCMakeArguments
                                                         : QStringList())
-                             + (forceExtraConfiguration ? m_parameters.extraCMakeArguments
-                                                        : QStringList());
+                             + (forceExtraConfiguration
+                                    ? (m_parameters.configurationChangesArguments
+                                       + m_parameters.additionalCMakeArguments)
+                                    : QStringList());
     qCDebug(cmakeFileApiMode) << "Parameters request these CMake arguments:" << args;
 
     const FilePath replyFile = FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory);
@@ -124,8 +128,9 @@ void FileApiReader::parse(bool forceCMakeRun,
     const bool hasArguments = !args.isEmpty();
     const bool replyFileMissing = !replyFile.exists();
     const bool cmakeFilesChanged = m_parameters.cmakeTool() && m_parameters.cmakeTool()->isAutoRun()
-                                   && anyOf(m_cmakeFiles, [&replyFile](const FilePath &f) {
-                                          return f.lastModified() > replyFile.lastModified();
+                                   && anyOf(m_cmakeFiles, [&replyFile](const CMakeFileInfo &info) {
+                                          return !info.isGenerated
+                                                 && info.path.lastModified() > replyFile.lastModified();
                                       });
     const bool queryFileChanged = anyOf(FileApiParser::cmakeQueryFilePaths(m_parameters.buildDirectory),
                                         [&replyFile](const FilePath &qf) {
@@ -150,7 +155,7 @@ void FileApiReader::parse(bool forceCMakeRun,
                                          .arg(args.join("\", \""));
         startCMakeState(args);
     } else {
-        endState(replyFile);
+        endState(replyFile, false);
     }
 }
 
@@ -168,6 +173,12 @@ void FileApiReader::stop()
     m_isParsing = false;
 }
 
+void FileApiReader::stopCMakeRun()
+{
+    if (m_cmakeProcess)
+        m_cmakeProcess->stop();
+}
+
 bool FileApiReader::isParsing() const
 {
     return m_isParsing;
@@ -175,7 +186,10 @@ bool FileApiReader::isParsing() const
 
 QSet<FilePath> FileApiReader::projectFilesToWatch() const
 {
-    return m_cmakeFiles;
+    return Utils::transform(
+                Utils::filtered(m_cmakeFiles,
+                                [](const CMakeFileInfo &info) { return !info.isGenerated; }),
+                [](const CMakeFileInfo &info) { return info.path;});
 }
 
 QList<CMakeBuildTarget> FileApiReader::takeBuildTargets(QString &errorMessage){
@@ -208,20 +222,6 @@ bool FileApiReader::usesAllCapsTargets() const
     return m_usesAllCapsTargets;
 }
 
-std::unique_ptr<CMakeProjectNode> FileApiReader::generateProjectTree(
-    const ProjectExplorer::TreeScanner::Result &allFiles,
-    QString &errorMessage,
-    bool includeHeaderNodes)
-{
-    Q_UNUSED(errorMessage)
-
-    if (includeHeaderNodes) {
-        addHeaderNodes(m_rootProjectNode.get(), m_knownHeaders, allFiles.allFiles);
-    }
-    addFileSystemNodes(m_rootProjectNode.get(), allFiles.folderNode);
-    return std::exchange(m_rootProjectNode, {});
-}
-
 RawProjectParts FileApiReader::createRawProjectParts(QString &errorMessage)
 {
     Q_UNUSED(errorMessage)
@@ -241,7 +241,7 @@ void FileApiReader::startState()
     emit configurationStarted();
 }
 
-void FileApiReader::endState(const FilePath &replyFilePath)
+void FileApiReader::endState(const FilePath &replyFilePath, bool restoredFromBackup)
 {
     qCDebug(cmakeFileApiMode) << "FileApiReader: END STATE.";
     QTC_ASSERT(m_isParsing, return );
@@ -249,37 +249,29 @@ void FileApiReader::endState(const FilePath &replyFilePath)
 
     const FilePath sourceDirectory = m_parameters.sourceDirectory;
     const FilePath buildDirectory = m_parameters.buildDirectory;
-    const FilePath topCmakeFile = m_cmakeFiles.size() == 1 ? *m_cmakeFiles.begin() : FilePath{};
     const QString cmakeBuildType = m_parameters.cmakeBuildType == "Build" ? "" : m_parameters.cmakeBuildType;
 
     QTC_CHECK(!replyFilePath.needsDevice());
     m_lastReplyTimestamp = replyFilePath.lastModified();
 
     m_future = runAsync(ProjectExplorerPlugin::sharedThreadPool(),
-                        [replyFilePath, sourceDirectory, buildDirectory, topCmakeFile, cmakeBuildType](
+                        [replyFilePath, sourceDirectory, buildDirectory, cmakeBuildType](
                             QFutureInterface<std::shared_ptr<FileApiQtcData>> &fi) {
                             auto result = std::make_shared<FileApiQtcData>();
                             FileApiData data = FileApiParser::parseData(fi,
                                                                         replyFilePath,
                                                                         cmakeBuildType,
                                                                         result->errorMessage);
-                            if (!result->errorMessage.isEmpty()) {
-                                *result = generateFallbackData(topCmakeFile,
-                                                               sourceDirectory,
-                                                               buildDirectory,
-                                                               result->errorMessage);
-                            } else {
+                            if (result->errorMessage.isEmpty())
                                 *result = extractData(data, sourceDirectory, buildDirectory);
-                            }
-                            if (!result->errorMessage.isEmpty()) {
+                            else
                                 qWarning() << result->errorMessage;
-                            }
 
                             fi.reportResult(result);
                         });
     onResultReady(m_future.value(),
                   this,
-                  [this, topCmakeFile, sourceDirectory, buildDirectory](
+                  [this, sourceDirectory, buildDirectory, restoredFromBackup](
                       const std::shared_ptr<FileApiQtcData> &value) {
                       m_isParsing = false;
                       m_cache = std::move(value->cache);
@@ -287,13 +279,12 @@ void FileApiReader::endState(const FilePath &replyFilePath)
                       m_buildTargets = std::move(value->buildTargets);
                       m_projectParts = std::move(value->projectParts);
                       m_rootProjectNode = std::move(value->rootProjectNode);
-                      m_knownHeaders = std::move(value->knownHeaders);
                       m_ctestPath = std::move(value->ctestPath);
                       m_isMultiConfig = std::move(value->isMultiConfig);
                       m_usesAllCapsTargets = std::move(value->usesAllCapsTargets);
 
                       if (value->errorMessage.isEmpty()) {
-                          emit this->dataAvailable();
+                          emit this->dataAvailable(restoredFromBackup);
                       } else {
                           emit this->errorOccurred(value->errorMessage);
                       }
@@ -335,12 +326,11 @@ void FileApiReader::writeConfigurationIntoBuildDirectory(const QStringList &conf
     QTC_CHECK(buildDir.ensureWritableDir());
 
     QByteArray contents;
+    QStringList unknownOptions;
     contents.append("# This file is managed by Qt Creator, do not edit!\n\n");
     contents.append(
-        transform(CMakeConfig::fromArguments(configurationArguments).toList(),
-            [](const CMakeConfigItem &item) {
-                return item.toCMakeSetLine(nullptr);
-            })
+        transform(CMakeConfig::fromArguments(configurationArguments, unknownOptions).toList(),
+                  [](const CMakeConfigItem &item) { return item.toCMakeSetLine(nullptr); })
             .join('\n')
             .toUtf8());
 
@@ -348,6 +338,21 @@ void FileApiReader::writeConfigurationIntoBuildDirectory(const QStringList &conf
     QTC_CHECK(settingsFile.writeFileContents(contents));
 }
 
+std::unique_ptr<CMakeProjectNode> FileApiReader::rootProjectNode()
+{
+    return std::exchange(m_rootProjectNode, {});
+}
+
+FilePath FileApiReader::topCmakeFile() const
+{
+    return m_cmakeFiles.size() == 1 ? (*m_cmakeFiles.begin()).path : FilePath{};
+}
+
+int FileApiReader::lastCMakeExitCode() const
+{
+    return m_lastCMakeExitCode;
+}
+
 void FileApiReader::startCMakeState(const QStringList &configurationArguments)
 {
     qCDebug(cmakeFileApiMode) << "FileApiReader: START CMAKE STATE.";
@@ -379,7 +384,8 @@ void FileApiReader::cmakeFinishedState()
 
     FileApiParser::setupCMakeFileApi(m_parameters.buildDirectory, m_watcher);
 
-    endState(FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory));
+    endState(FileApiParser::scanForCMakeReplyFile(m_parameters.buildDirectory),
+             m_lastCMakeExitCode != 0);
 }
 
 void FileApiReader::replyDirectoryHasChanged(const QString &directory) const
