@@ -45,9 +45,9 @@
 #include "qmt/project/project.h"
 
 #include <extensionsystem/pluginmanager.h>
-#include <cpptools/cppclassesfilter.h>
-#include <cpptools/indexitem.h>
-#include <cpptools/searchsymbols.h>
+#include <cppeditor/cpplocatorfilter.h>
+#include <cppeditor/indexitem.h>
+#include <cppeditor/searchsymbols.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/locator/ilocatorfilter.h>
 #include <utils/qtcassert.h>
@@ -93,7 +93,7 @@ void ElementTasks::openElement(const qmt::MElement *element)
 
 void ElementTasks::openElement(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     OpenDiagramElementVisitor visitor;
     visitor.setModelController(d->documentController->modelController());
@@ -109,15 +109,15 @@ bool ElementTasks::hasClassDefinition(const qmt::MElement *element) const
                 : klass->umlNamespace() + "::" + klass->name();
 
         Core::ILocatorFilter *classesFilter
-                = CppTools::CppModelManager::instance()->classesFilter();
+                = CppEditor::CppModelManager::instance()->classesFilter();
         if (!classesFilter)
             return false;
 
         QFutureInterface<Core::LocatorFilterEntry> dummyInterface;
-        QList<Core::LocatorFilterEntry> matches = classesFilter->matchesFor(dummyInterface,
-                                                                            qualifiedClassName);
-        foreach (const Core::LocatorFilterEntry &entry, matches) {
-            CppTools::IndexItem::Ptr info = qvariant_cast<CppTools::IndexItem::Ptr>(entry.internalData);
+        const QList<Core::LocatorFilterEntry> matches
+            = classesFilter->matchesFor(dummyInterface, qualifiedClassName);
+        for (const Core::LocatorFilterEntry &entry : matches) {
+            CppEditor::IndexItem::Ptr info = qvariant_cast<CppEditor::IndexItem::Ptr>(entry.internalData);
             if (info->scopedSymbolName() != qualifiedClassName)
                 continue;
             return true;
@@ -129,7 +129,7 @@ bool ElementTasks::hasClassDefinition(const qmt::MElement *element) const
 bool ElementTasks::hasClassDefinition(const qmt::DElement *element,
                                       const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(
                 element->modelUid());
@@ -146,25 +146,28 @@ void ElementTasks::openClassDefinition(const qmt::MElement *element)
                 : klass->umlNamespace() + "::" + klass->name();
 
         Core::ILocatorFilter *classesFilter
-                = CppTools::CppModelManager::instance()->classesFilter();
+                = CppEditor::CppModelManager::instance()->classesFilter();
         if (!classesFilter)
             return;
 
         QFutureInterface<Core::LocatorFilterEntry> dummyInterface;
-        QList<Core::LocatorFilterEntry> matches = classesFilter->matchesFor(dummyInterface, qualifiedClassName);
-        foreach (const Core::LocatorFilterEntry &entry, matches) {
-            CppTools::IndexItem::Ptr info = qvariant_cast<CppTools::IndexItem::Ptr>(entry.internalData);
+        const QList<Core::LocatorFilterEntry> matches
+            = classesFilter->matchesFor(dummyInterface, qualifiedClassName);
+        for (const Core::LocatorFilterEntry &entry : matches) {
+            CppEditor::IndexItem::Ptr info = qvariant_cast<CppEditor::IndexItem::Ptr>(entry.internalData);
             if (info->scopedSymbolName() != qualifiedClassName)
                 continue;
-            if (Core::EditorManager::instance()->openEditorAt(info->fileName(), info->line(), info->column()))
+            if (Core::EditorManager::instance()->openEditorAt(
+                    {Utils::FilePath::fromString(info->fileName()), info->line(), info->column()})) {
                 return;
+            }
         }
     }
 }
 
 void ElementTasks::openClassDefinition(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -175,13 +178,13 @@ void ElementTasks::openClassDefinition(const qmt::DElement *element, const qmt::
 bool ElementTasks::hasHeaderFile(const qmt::MElement *element) const
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
     return false;
 }
 
 bool ElementTasks::hasHeaderFile(const qmt::DElement *element, const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -192,13 +195,13 @@ bool ElementTasks::hasHeaderFile(const qmt::DElement *element, const qmt::MDiagr
 bool ElementTasks::hasSourceFile(const qmt::MElement *element) const
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
     return false;
 }
 
 bool ElementTasks::hasSourceFile(const qmt::DElement *element, const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -209,12 +212,12 @@ bool ElementTasks::hasSourceFile(const qmt::DElement *element, const qmt::MDiagr
 void ElementTasks::openHeaderFile(const qmt::MElement *element)
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
 }
 
 void ElementTasks::openHeaderFile(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -225,12 +228,12 @@ void ElementTasks::openHeaderFile(const qmt::DElement *element, const qmt::MDiag
 void ElementTasks::openSourceFile(const qmt::MElement *element)
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
 }
 
 void ElementTasks::openSourceFile(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -241,13 +244,13 @@ void ElementTasks::openSourceFile(const qmt::DElement *element, const qmt::MDiag
 bool ElementTasks::hasFolder(const qmt::MElement *element) const
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
     return false;
 }
 
 bool ElementTasks::hasFolder(const qmt::DElement *element, const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -258,12 +261,12 @@ bool ElementTasks::hasFolder(const qmt::DElement *element, const qmt::MDiagram *
 void ElementTasks::showFolder(const qmt::MElement *element)
 {
     // TODO implement
-    Q_UNUSED(element);
+    Q_UNUSED(element)
 }
 
 void ElementTasks::showFolder(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -281,7 +284,7 @@ bool ElementTasks::hasDiagram(const qmt::MElement *element) const
 
 bool ElementTasks::hasDiagram(const qmt::DElement *element, const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -303,7 +306,7 @@ void ElementTasks::openDiagram(const qmt::MElement *element)
 
 void ElementTasks::openDiagram(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -330,7 +333,7 @@ bool ElementTasks::hasParentDiagram(const qmt::MElement *element) const
 
 bool ElementTasks::hasParentDiagram(const qmt::DElement *element, const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     if (!element)
         return false;
@@ -362,7 +365,7 @@ void ElementTasks::openParentDiagram(const qmt::MElement *element)
 
 void ElementTasks::openParentDiagram(const qmt::DElement *element, const qmt::MElement *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     if (!element)
         return;
@@ -381,7 +384,7 @@ bool ElementTasks::mayCreateDiagram(const qmt::MElement *element) const
 bool ElementTasks::mayCreateDiagram(const qmt::DElement *element,
                                     const qmt::MDiagram *diagram) const
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
@@ -414,7 +417,7 @@ void ElementTasks::createAndOpenDiagram(const qmt::MElement *element)
 
 void ElementTasks::createAndOpenDiagram(const qmt::DElement *element, const qmt::MDiagram *diagram)
 {
-    Q_UNUSED(diagram);
+    Q_UNUSED(diagram)
 
     qmt::MElement *melement = d->documentController->modelController()->findElement(element->modelUid());
     if (!melement)
