@@ -39,26 +39,28 @@
 #include "projectexplorerconstants.h"
 #include "projectexplorer.h"
 #include "project.h"
-#include "projectmacroexpander.h"
 #include "projecttree.h"
 #include "session.h"
 #include "target.h"
 #include "toolchain.h"
 
+#include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 
+#include <projectexplorer/devicesupport/idevice.h>
+
 #include <utils/algorithm.h>
 #include <utils/detailswidget.h>
-#include <utils/macroexpander.h>
-#include <utils/mimetypes/mimedatabase.h>
-#include <utils/mimetypes/mimetype.h>
 #include <utils/layoutbuilder.h>
+#include <utils/macroexpander.h>
+#include <utils/mimeutils.h>
 #include <utils/qtcassert.h>
 #include <utils/variablechooser.h>
 
 #include <QCheckBox>
 #include <QDebug>
 #include <QFormLayout>
+#include <QLoggingCategory>
 #include <QVBoxLayout>
 
 using namespace Utils;
@@ -68,6 +70,9 @@ const char BUILD_STEP_LIST_PREFIX[] = "ProjectExplorer.BuildConfiguration.BuildS
 const char CLEAR_SYSTEM_ENVIRONMENT_KEY[] = "ProjectExplorer.BuildConfiguration.ClearSystemEnvironment";
 const char USER_ENVIRONMENT_CHANGES_KEY[] = "ProjectExplorer.BuildConfiguration.UserEnvironmentChanges";
 const char CUSTOM_PARSERS_KEY[] = "ProjectExplorer.BuildConfiguration.CustomParsers";
+const char PARSE_STD_OUT_KEY[] = "ProjectExplorer.BuildConfiguration.ParseStandardOutput";
+
+Q_LOGGING_CATEGORY(bcLog, "qtc.buildconfig", QtWarningMsg)
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -116,9 +121,17 @@ class CustomParsersBuildWidget : public NamedWidget
 public:
     CustomParsersBuildWidget(BuildConfiguration *bc) : NamedWidget(tr("Custom Output Parsers"))
     {
-        const auto selectionWidget = new CustomParsersSelectionWidget(this);
         const auto layout = new QVBoxLayout(this);
         layout->setContentsMargins(0, 0, 0, 0);
+
+        const auto pasteStdOutCB = new QCheckBox(tr("Parse standard output during build"), this);
+        pasteStdOutCB->setToolTip(tr("Makes output parsers look for diagnostics "
+                                     "on stdout rather than stderr."));
+        pasteStdOutCB->setChecked(bc->parseStdOut());
+        layout->addWidget(pasteStdOutCB);
+
+        connect(pasteStdOutCB, &QCheckBox::clicked, bc, &BuildConfiguration::setParseStdOut);
+        const auto selectionWidget = new CustomParsersSelectionWidget(this);
         layout->addWidget(selectionWidget);
 
         connect(selectionWidget, &CustomParsersSelectionWidget::selectionChanged,
@@ -143,6 +156,7 @@ public:
     BuildStepList m_buildSteps;
     BuildStepList m_cleanSteps;
     BuildDirectoryAspect *m_buildDirectoryAspect = nullptr;
+    StringAspect *m_tooltipAspect = nullptr;
     FilePath m_lastEmittedBuildDirectory;
     mutable Environment m_cachedEnvironment;
     QString m_configWidgetDisplayName;
@@ -150,6 +164,7 @@ public:
     QList<Utils::Id> m_initialBuildSteps;
     QList<Utils::Id> m_initialCleanSteps;
     Utils::MacroExpander m_macroExpander;
+    bool m_parseStdOut = false;
     QList<Utils::Id> m_customParsers;
 
     // FIXME: Remove.
@@ -186,28 +201,38 @@ BuildConfiguration::BuildConfiguration(Target *target, Utils::Id id)
                              tr("Variables in the build configuration's environment"),
                              [this](const QString &var) { return environment().expandedValueForKey(var); });
 
-    updateCacheAndEmitEnvironmentChanged();
+    connect(Core::ICore::instance(), &Core::ICore::systemEnvironmentChanged,
+            this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     connect(target, &Target::kitChanged,
             this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     connect(this, &BuildConfiguration::environmentChanged,
             this, &BuildConfiguration::emitBuildDirectoryChanged);
     connect(target->project(), &Project::environmentChanged,
-            this, &BuildConfiguration::environmentChanged);
+            this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
     // Many macroexpanders are based on the current project, so they may change the environment:
     connect(ProjectTree::instance(), &ProjectTree::currentProjectChanged,
             this, &BuildConfiguration::updateCacheAndEmitEnvironmentChanged);
 
     d->m_buildDirectoryAspect = addAspect<BuildDirectoryAspect>(this);
     d->m_buildDirectoryAspect->setBaseFileName(target->project()->projectDirectory());
-    d->m_buildDirectoryAspect->setEnvironment(environment());
+    d->m_buildDirectoryAspect->setEnvironmentChange(EnvironmentChange::fromFixedEnvironment(environment()));
     d->m_buildDirectoryAspect->setMacroExpanderProvider([this] { return macroExpander(); });
     connect(d->m_buildDirectoryAspect, &StringAspect::changed,
             this, &BuildConfiguration::emitBuildDirectoryChanged);
     connect(this, &BuildConfiguration::environmentChanged, this, [this] {
-        d->m_buildDirectoryAspect->setEnvironment(environment());
+        d->m_buildDirectoryAspect->setEnvironmentChange(EnvironmentChange::fromFixedEnvironment(environment()));
         emit this->target()->buildEnvironmentChanged(this);
     });
 
+    d->m_tooltipAspect = addAspect<StringAspect>();
+    d->m_tooltipAspect->setLabelText(tr("Tooltip in target selector:"));
+    d->m_tooltipAspect->setToolTip(tr("Appears as a tooltip when hovering the build configuration"));
+    d->m_tooltipAspect->setDisplayStyle(StringAspect::LineEditDisplay);
+    d->m_tooltipAspect->setSettingsKey("ProjectExplorer.BuildConfiguration.Tooltip");
+    connect(d->m_tooltipAspect, &StringAspect::changed, this, [this] {
+        setToolTip(d->m_tooltipAspect->value());
+    });
+
     connect(target, &Target::parsingStarted, this, &BuildConfiguration::enabledChanged);
     connect(target, &Target::parsingFinished, this, &BuildConfiguration::enabledChanged);
     connect(this, &BuildConfiguration::enabledChanged, this, [this] {
@@ -225,9 +250,14 @@ BuildConfiguration::~BuildConfiguration()
 
 FilePath BuildConfiguration::buildDirectory() const
 {
-    QString path = environment().expandVariables(d->m_buildDirectoryAspect->value().trimmed());
-    path = QDir::cleanPath(macroExpander()->expand(path));
-    return FilePath::fromString(QDir::cleanPath(QDir(target()->project()->projectDirectory().toString()).absoluteFilePath(path)));
+    FilePath path = FilePath::fromUserInput(
+        environment().expandVariables(d->m_buildDirectoryAspect->value().trimmed()));
+    path = macroExpander()->expand(path);
+    path = path.cleanPath();
+
+    const FilePath projectDir = target()->project()->projectDirectory();
+
+    return projectDir.resolvePath(path);
 }
 
 FilePath BuildConfiguration::rawBuildDirectory() const
@@ -240,6 +270,9 @@ void BuildConfiguration::setBuildDirectory(const FilePath &dir)
     if (dir == d->m_buildDirectoryAspect->filePath())
         return;
     d->m_buildDirectoryAspect->setFilePath(dir);
+    const FilePath fixedDir = BuildDirectoryAspect::fixupDir(buildDirectory());
+    if (!fixedDir.isEmpty())
+        d->m_buildDirectoryAspect->setFilePath(fixedDir);
     emitBuildDirectoryChanged();
 }
 
@@ -251,13 +284,15 @@ void BuildConfiguration::addConfigWidgets(const std::function<void(NamedWidget *
     adder(new Internal::BuildStepListWidget(buildSteps()));
     adder(new Internal::BuildStepListWidget(cleanSteps()));
 
-    QList<NamedWidget *> subConfigWidgets = createSubConfigWidgets();
-    foreach (NamedWidget *subConfigWidget, subConfigWidgets)
+    const QList<NamedWidget *> subConfigWidgets = createSubConfigWidgets();
+    for (NamedWidget *subConfigWidget : subConfigWidgets)
         adder(subConfigWidget);
 }
 
 void BuildConfiguration::doInitialize(const BuildInfo &info)
 {
+    updateCacheAndEmitEnvironmentChanged();
+
     setDisplayName(info.displayName);
     setDefaultDisplayName(info.displayName);
     setBuildDirectory(info.buildDirectory);
@@ -270,8 +305,6 @@ void BuildConfiguration::doInitialize(const BuildInfo &info)
     for (Utils::Id id : qAsConst(d->m_initialCleanSteps))
         d->m_cleanSteps.appendStep(id);
 
-    acquaintAspects();
-
     if (d->m_initializer)
         d->m_initializer(info);
 }
@@ -283,8 +316,7 @@ MacroExpander *BuildConfiguration::macroExpander() const
 
 bool BuildConfiguration::createBuildDirectory()
 {
-    QDir dir;
-    const auto result = dir.mkpath(buildDirectory().toString());
+    const bool result = buildDirectory().ensureWritableDir();
     buildDirectoryAspect()->validateInput();
     return result;
 }
@@ -313,11 +345,12 @@ NamedWidget *BuildConfiguration::createConfigWidget()
         widget = named;
     }
 
-    LayoutBuilder builder(widget);
+    Layouting::Form builder;
     for (BaseAspect *aspect : aspects()) {
         if (aspect->isVisible())
             aspect->addToLayout(builder.finishRow());
     }
+    builder.attachTo(widget, false);
 
     return named;
 }
@@ -367,6 +400,7 @@ QVariantMap BuildConfiguration::toMap() const
     map.insert(QLatin1String(BUILD_STEP_LIST_PREFIX) + QString::number(0), d->m_buildSteps.toMap());
     map.insert(QLatin1String(BUILD_STEP_LIST_PREFIX) + QString::number(1), d->m_cleanSteps.toMap());
 
+    map.insert(PARSE_STD_OUT_KEY, d->m_parseStdOut);
     map.insert(CUSTOM_PARSERS_KEY, transform(d->m_customParsers,&Utils::Id::toSetting));
 
     return map;
@@ -401,9 +435,12 @@ bool BuildConfiguration::fromMap(const QVariantMap &map)
         }
     }
 
+    d->m_parseStdOut = map.value(PARSE_STD_OUT_KEY).toBool();
     d->m_customParsers = transform(map.value(CUSTOM_PARSERS_KEY).toList(), &Utils::Id::fromSetting);
 
-    return ProjectConfiguration::fromMap(map);
+    const bool res = ProjectConfiguration::fromMap(map);
+    setToolTip(d->m_tooltipAspect->value());
+    return res;
 }
 
 void BuildConfiguration::updateCacheAndEmitEnvironmentChanged()
@@ -452,10 +489,12 @@ void BuildConfiguration::setBuildDirectorySettingsKey(const QString &key)
 Environment BuildConfiguration::baseEnvironment() const
 {
     Environment result;
-    if (useSystemEnvironment())
-        result = Environment::systemEnvironment();
+    if (useSystemEnvironment()) {
+        ProjectExplorer::IDevice::ConstPtr devicePtr = BuildDeviceKitAspect::device(kit());
+        result = devicePtr ? devicePtr->systemEnvironment() : Environment::systemEnvironment();
+    }
     addToEnvironment(result);
-    kit()->addToEnvironment(result);
+    kit()->addToBuildEnvironment(result);
     result.modify(project()->additionalEnvironment());
     return result;
 }
@@ -496,6 +535,9 @@ void BuildConfiguration::setCustomParsers(const QList<Utils::Id> &parsers)
     d->m_customParsers = parsers;
 }
 
+bool BuildConfiguration::parseStdOut() const { return d->m_parseStdOut; }
+void BuildConfiguration::setParseStdOut(bool b) { d->m_parseStdOut = b; }
+
 bool BuildConfiguration::useSystemEnvironment() const
 {
     return !d->m_clearSystemEnvironment;
@@ -516,13 +558,11 @@ void BuildConfiguration::setUserEnvironmentChanges(const EnvironmentItems &diff)
 
 bool BuildConfiguration::isEnabled() const
 {
-    return !buildSystem()->isParsing() && buildSystem()->hasParsingData();
+    return buildSystem()->hasParsingData();
 }
 
 QString BuildConfiguration::disabledReason() const
 {
-    if (buildSystem()->isParsing())
-        return (tr("The project is currently being parsed."));
     if (!buildSystem()->hasParsingData())
         return (tr("The project was not parsed successfully."));
     return QString();
@@ -564,24 +604,59 @@ bool BuildConfiguration::isActive() const
     return target()->isActive() && target()->activeBuildConfiguration() == this;
 }
 
-/*!
- * Helper function that prepends the directory containing the C++ toolchain to
- * PATH. This is used to in build configurations targeting broken build systems
- * to provide hints about which compiler to use.
- */
-
-void BuildConfiguration::prependCompilerPathToEnvironment(Kit *k, Environment &env)
+FilePath BuildConfiguration::buildDirectoryFromTemplate(const FilePath &projectDir,
+                                                        const FilePath &mainFilePath,
+                                                        const QString &projectName,
+                                                        const Kit *kit,
+                                                        const QString &bcName,
+                                                        BuildType buildType,
+                                                        const QString &buildSystem)
 {
-    const ToolChain *tc = ToolChainKitAspect::cxxToolChain(k);
+    MacroExpander exp;
 
-    if (!tc)
-        return;
+    qCDebug(bcLog) << Q_FUNC_INFO << projectDir << mainFilePath << projectName << bcName;
 
-    const FilePath compilerDir = tc->compilerCommand().parentDir();
-    if (!compilerDir.isEmpty())
-        env.prependOrSetPath(compilerDir.toString());
+    // TODO: Remove "Current" variants in ~4.16
+    exp.registerFileVariables(Constants::VAR_CURRENTPROJECT_PREFIX,
+                              QCoreApplication::translate("ProjectExplorer", "Main file of current project"),
+                              [mainFilePath] { return mainFilePath; }, false);
+    exp.registerFileVariables("Project",
+                              QCoreApplication::translate("ProjectExplorer", "Main file of the project"),
+                              [mainFilePath] { return mainFilePath; });
+    exp.registerVariable(Constants::VAR_CURRENTPROJECT_NAME,
+                         QCoreApplication::translate("ProjectExplorer", "Name of current project"),
+                         [projectName] { return projectName; }, false);
+    exp.registerVariable("Project:Name",
+                         QCoreApplication::translate("ProjectExplorer", "Name of the project"),
+                         [projectName] { return projectName; });
+    exp.registerVariable(Constants::VAR_CURRENTBUILD_NAME,
+                         QCoreApplication::translate("ProjectExplorer", "Name of current build"),
+                         [bcName] { return bcName; }, false);
+    exp.registerVariable("BuildConfig:Name",
+                         QCoreApplication::translate(
+                             "ProjectExplorer", "Name of the project's active build configuration"),
+                         [bcName] { return bcName; });
+    exp.registerVariable("BuildSystem:Name",
+                         QCoreApplication::translate(
+                             "ProjectExplorer", "Name of the project's active build system"),
+                         [buildSystem] { return buildSystem; });
+    exp.registerVariable("CurrentBuild:Type",
+                         QCoreApplication::translate("ProjectExplorer", "Type of current build"),
+                         [buildType] { return buildTypeName(buildType); }, false);
+    exp.registerVariable("BuildConfig:Type",
+                         QCoreApplication::translate(
+                             "ProjectExplorer", "Type of the project's active build configuration"),
+                         [buildType] { return buildTypeName(buildType); });
+    exp.registerSubProvider([kit] { return kit->macroExpander(); });
+
+    QString buildDir = ProjectExplorerPlugin::buildDirectoryTemplate();
+    qCDebug(bcLog) << "build dir template:" << buildDir;
+    buildDir = exp.expand(buildDir);
+    qCDebug(bcLog) << "expanded build:" << buildDir;
+    buildDir.replace(" ", "-");
+
+    return projectDir.resolvePath(buildDir);
 }
-
 ///
 // IBuildConfigurationFactory
 ///
@@ -642,10 +717,9 @@ BuildConfigurationFactory *BuildConfigurationFactory::find(const Kit *k, const F
 {
     QTC_ASSERT(k, return nullptr);
     const Utils::Id deviceType = DeviceTypeKitAspect::deviceTypeId(k);
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
-        if (Utils::mimeTypeForFile(projectPath.toString())
-                .matchesName(factory->m_supportedProjectMimeTypeName)
-                && factory->supportsTargetDeviceType(deviceType))
+    for (BuildConfigurationFactory *factory : qAsConst(g_buildConfigurationFactories)) {
+        if (Utils::mimeTypeForFile(projectPath).matchesName(factory->m_supportedProjectMimeTypeName)
+            && factory->supportsTargetDeviceType(deviceType))
             return factory;
     }
     return nullptr;
@@ -654,7 +728,7 @@ BuildConfigurationFactory *BuildConfigurationFactory::find(const Kit *k, const F
 // create
 BuildConfigurationFactory * BuildConfigurationFactory::find(Target *parent)
 {
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
+    for (BuildConfigurationFactory *factory : qAsConst(g_buildConfigurationFactories)) {
         if (factory->canHandle(parent))
             return factory;
     }
@@ -716,14 +790,13 @@ BuildConfiguration *BuildConfigurationFactory::create(Target *parent, const Buil
 BuildConfiguration *BuildConfigurationFactory::restore(Target *parent, const QVariantMap &map)
 {
     const Utils::Id id = idFromMap(map);
-    for (BuildConfigurationFactory *factory : g_buildConfigurationFactories) {
+    for (BuildConfigurationFactory *factory : qAsConst(g_buildConfigurationFactories)) {
         QTC_ASSERT(factory->m_creator, return nullptr);
         if (!factory->canHandle(parent))
             continue;
         if (!id.name().startsWith(factory->m_buildConfigId.name()))
             continue;
         BuildConfiguration *bc = factory->m_creator(parent);
-        bc->acquaintAspects();
         QTC_ASSERT(bc, return nullptr);
         if (!bc->fromMap(map)) {
             delete bc;
