@@ -61,14 +61,14 @@ static inline QString msgAppNotFound(const QString &id)
 }
 
 // -- Commands and helpers
-static QString linguistBinary(const QtSupport::BaseQtVersion *qtVersion)
+static QString linguistBinary(const QtSupport::QtVersion *qtVersion)
 {
     if (qtVersion)
         return qtVersion->linguistFilePath().toString();
     return QLatin1String(Utils::HostOsInfo::isMacHost() ? "Linguist" : "linguist");
 }
 
-static QString designerBinary(const QtSupport::BaseQtVersion *qtVersion)
+static QString designerBinary(const QtSupport::QtVersion *qtVersion)
 {
     if (qtVersion)
         return qtVersion->designerFilePath().toString();
@@ -99,12 +99,12 @@ static const char linguistDisplayName[] = QT_TRANSLATE_NOOP("OpenWith::Editors",
 ExternalQtEditor::ExternalQtEditor(Utils::Id id,
                                    const QString &displayName,
                                    const QString &mimetype,
-                                   const CommandForQtVersion &commandForQtVersion) :
-    m_mimeTypes(mimetype),
-    m_id(id),
-    m_displayName(displayName),
-    m_commandForQtVersion(commandForQtVersion)
+                                   const CommandForQtVersion &commandForQtVersion)
+    : m_commandForQtVersion(commandForQtVersion)
 {
+    setId(id);
+    setDisplayName(displayName);
+    setMimeTypes({mimetype});
 }
 
 ExternalQtEditor *ExternalQtEditor::createLinguistEditor()
@@ -127,25 +127,10 @@ ExternalQtEditor *ExternalQtEditor::createDesignerEditor()
     }
 }
 
-QStringList ExternalQtEditor::mimeTypes() const
-{
-    return m_mimeTypes;
-}
-
-Utils::Id ExternalQtEditor::id() const
-{
-    return m_id;
-}
-
-QString ExternalQtEditor::displayName() const
-{
-    return m_displayName;
-}
-
-static QString findFirstCommand(QVector<QtSupport::BaseQtVersion *> qtVersions,
+static QString findFirstCommand(QVector<QtSupport::QtVersion *> qtVersions,
                                 ExternalQtEditor::CommandForQtVersion command)
 {
-    foreach (QtSupport::BaseQtVersion *qt, qtVersions) {
+    foreach (QtSupport::QtVersion *qt, qtVersions) {
         if (qt) {
             const QString binary = command(qt);
             if (!binary.isEmpty())
@@ -166,7 +151,7 @@ bool ExternalQtEditor::getEditorLaunchData(const Utils::FilePath &filePath,
     // - any other kit
     // As fallback check PATH
     data->workingDirectory.clear();
-    QVector<QtSupport::BaseQtVersion *> qtVersionsToCheck; // deduplicated after being filled
+    QVector<QtSupport::QtVersion *> qtVersionsToCheck; // deduplicated after being filled
     if (const Project *project = SessionManager::projectForFile(filePath)) {
         data->workingDirectory = project->projectDirectory();
         // active kit
@@ -185,8 +170,11 @@ bool ExternalQtEditor::getEditorLaunchData(const Utils::FilePath &filePath,
     qtVersionsToCheck = Utils::filteredUnique(qtVersionsToCheck); // can still contain nullptr
     data->binary = findFirstCommand(qtVersionsToCheck, m_commandForQtVersion);
     // fallback
-    if (data->binary.isEmpty())
-        data->binary = Utils::QtcProcess::locateBinary(m_commandForQtVersion(nullptr));
+    if (data->binary.isEmpty()) {
+        const QString path = qEnvironmentVariable("PATH");
+        data->binary = Utils::QtcProcess::locateBinary(path, m_commandForQtVersion(nullptr));
+    }
+
     if (data->binary.isEmpty()) {
         *errorMessage = msgAppNotFound(id().toString());
         return false;
@@ -287,12 +275,7 @@ bool DesignerExternalEditor::startEditor(const Utils::FilePath &filePath, QStrin
         m_processCache.insert(binary, socket);
         auto mapSlot = [this, binary] { processTerminated(binary); };
         connect(socket, &QAbstractSocket::disconnected, this, mapSlot);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        const auto errorOccurred = QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error);
-#else
-        const auto errorOccurred = &QAbstractSocket::errorOccurred;
-#endif
-        connect(socket, errorOccurred, this, mapSlot);
+        connect(socket, &QAbstractSocket::errorOccurred, this, mapSlot);
     }
     return true;
 }
