@@ -24,6 +24,7 @@
 ****************************************************************************/
 
 #include "modelnodeoperations.h"
+#include "designmodewidget.h"
 #include "modelnodecontextmenu_helper.h"
 #include "addimagesdialog.h"
 #include "layoutingridlayout.h"
@@ -51,7 +52,7 @@
 #include <componentcore_constants.h>
 #include <stylesheetmerger.h>
 
-#include <limits>
+#include <designermcumanager.h>
 #include <qmldesignerplugin.h>
 
 #include <coreplugin/messagebox.h>
@@ -82,10 +83,15 @@
 #include <QPushButton>
 #include <QGridLayout>
 #include <QPointer>
+#include <QMessageBox>
+#include <QPair>
 
 #include <algorithm>
 #include <functional>
 #include <cmath>
+#include <limits>
+
+#include <bindingeditor/signallist.h>
 
 namespace QmlDesigner {
 
@@ -109,7 +115,7 @@ static inline void reparentTo(const ModelNode &node, const QmlItemNode &parent)
 static inline QPointF getUpperLeftPosition(const QList<ModelNode> &modelNodeList)
 {
     QPointF postion(std::numeric_limits<qreal>::max(), std::numeric_limits<qreal>::max());
-    foreach (const ModelNode &modelNode, modelNodeList) {
+    for (const ModelNode &modelNode : modelNodeList) {
         if (QmlItemNode::isValidQmlItemNode(modelNode)) {
             QmlItemNode qmlIitemNode = QmlItemNode(modelNode);
             if (qmlIitemNode.instancePosition().x() < postion.x())
@@ -146,7 +152,8 @@ void deSelect(const SelectionContext &selectionState)
 {
     if (selectionState.view()) {
         QList<ModelNode> selectedNodes = selectionState.view()->selectedModelNodes();
-        foreach (const ModelNode &node, selectionState.selectedModelNodes()) {
+        const QList<ModelNode> nodes = selectionState.selectedModelNodes();
+        for (const ModelNode &node : nodes) {
             if (selectedNodes.contains(node))
                 selectedNodes.removeAll(node);
         }
@@ -183,7 +190,7 @@ void toFront(const SelectionContext &selectionState)
             if (index != lastIndex)
                 parentProperty.slide(index, lastIndex);
         }
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -204,7 +211,7 @@ void toBack(const SelectionContext &selectionState)
                 parentProperty.slide(index, 0);
         }
 
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -268,7 +275,7 @@ void setVisible(const SelectionContext &selectionState)
 
     try {
         selectionState.selectedModelNodes().constFirst().variantProperty("visible").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -281,7 +288,7 @@ void setFillWidth(const SelectionContext &selectionState)
 
     try {
         selectionState.firstSelectedModelNode().variantProperty("Layout.fillWidth").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -294,7 +301,7 @@ void setFillHeight(const SelectionContext &selectionState)
 
     try {
         selectionState.firstSelectedModelNode().variantProperty("Layout.fillHeight").setValue(selectionState.toggled());
-    } catch (const RewritingException &e) { //better save then sorry
+    } catch (const RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -305,7 +312,8 @@ void resetSize(const SelectionContext &selectionState)
         return;
 
     selectionState.view()->executeInTransaction("DesignerActionManager|resetSize",[selectionState](){
-        foreach (ModelNode node, selectionState.selectedModelNodes()) {
+        const QList<ModelNode> nodes = selectionState.selectedModelNodes();
+        for (const ModelNode &node : nodes) {
             QmlItemNode itemNode(node);
             if (itemNode.isValid()) {
                 itemNode.removeProperty("width");
@@ -321,7 +329,8 @@ void resetPosition(const SelectionContext &selectionState)
         return;
 
     selectionState.view()->executeInTransaction("DesignerActionManager|resetPosition",[selectionState](){
-        foreach (ModelNode node, selectionState.selectedModelNodes()) {
+        const QList<ModelNode> nodes = selectionState.selectedModelNodes();
+        for (const ModelNode &node : nodes) {
             QmlItemNode itemNode(node);
             if (itemNode.isValid()) {
                 itemNode.removeProperty("x");
@@ -469,19 +478,16 @@ static void layoutHelperFunction(const SelectionContext &selectionContext,
         const QmlItemNode qmlItemNode = QmlItemNode(selectionContext.firstSelectedModelNode());
 
         if (qmlItemNode.hasInstanceParentItem()) {
-            ModelNode layoutNode;
-            selectionContext.view()->executeInTransaction("DesignerActionManager|layoutHelperFunction1",[=, &layoutNode](){
+
+            selectionContext.view()->executeInTransaction("DesignerActionManager|layoutHelperFunction",[=](){
 
                 QmlItemNode parentNode = qmlItemNode.instanceParentItem();
 
                 NodeMetaInfo metaInfo = selectionContext.view()->model()->metaInfo(layoutType);
 
-                layoutNode = selectionContext.view()->createModelNode(layoutType, metaInfo.majorVersion(), metaInfo.minorVersion());
+                const ModelNode layoutNode = selectionContext.view()->createModelNode(layoutType, metaInfo.majorVersion(), metaInfo.minorVersion());
 
                 reparentTo(layoutNode, parentNode);
-            });
-
-            selectionContext.view()->executeInTransaction("DesignerActionManager|layoutHelperFunction2",[=](){
 
                 QList<ModelNode> sortedSelectedNodes =  selectionContext.selectedModelNodes();
                 Utils::sort(sortedSelectedNodes, lessThan);
@@ -518,9 +524,14 @@ void layoutFlowPositioner(const SelectionContext &selectionContext)
 void layoutRowLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        layoutHelperFunction(selectionContext, "QtQuick.Layouts.RowLayout", compareByX);
-    } catch (RewritingException &e) { //better save then sorry
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            layoutHelperFunction(selectionContext, "QtQuick.Row", compareByX);
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            layoutHelperFunction(selectionContext, "QtQuick.Layouts.RowLayout", compareByX);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -528,9 +539,14 @@ void layoutRowLayout(const SelectionContext &selectionContext)
 void layoutColumnLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        layoutHelperFunction(selectionContext, "QtQuick.Layouts.ColumnLayout", compareByY);
-    } catch (RewritingException &e) { //better save then sorry
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            layoutHelperFunction(selectionContext, "QtQuick.Column", compareByX);
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            layoutHelperFunction(selectionContext, "QtQuick.Layouts.ColumnLayout", compareByY);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -538,9 +554,16 @@ void layoutColumnLayout(const SelectionContext &selectionContext)
 void layoutGridLayout(const SelectionContext &selectionContext)
 {
     try {
-        LayoutInGridLayout::ensureLayoutImport(selectionContext);
-        LayoutInGridLayout::layout(selectionContext);
-    } catch (RewritingException &e) { //better save then sorry
+        Q_ASSERT(!DesignerMcuManager::instance().isMCUProject()); //remove this line when grids are finally supported
+
+        if (DesignerMcuManager::instance().isMCUProject()) {
+            //qt for mcu doesn't support any grids yet
+        }
+        else {
+            LayoutInGridLayout::ensureLayoutImport(selectionContext);
+            LayoutInGridLayout::layout(selectionContext);
+        }
+    } catch (RewritingException &e) { //better safe than sorry
         e.showException();
     }
 }
@@ -580,8 +603,8 @@ static void addSignal(const QString &typeName, const QString &itemId, const QStr
         signalHandlerName = "on" + toUpper(signalName).toUtf8();
     else
         signalHandlerName = itemId.toUtf8() + ".on" + toUpper(signalName).toUtf8();
-
-    foreach (const ModelNode &modelNode, rewriterView.allModelNodes()) {
+    const QList<ModelNode> nodes = rewriterView.allModelNodes();
+    for (const ModelNode &modelNode : nodes) {
         if (modelNode.type() == typeName.toUtf8()) {
             modelNode.signalHandlerProperty(signalHandlerName).setSource(QLatin1String("{\n}"));
         }
@@ -592,7 +615,7 @@ static QStringList cleanSignalNames(const QStringList &input)
 {
     QStringList output;
 
-    foreach (const QString &signal, input)
+    for (const QString &signal : input)
         if (!signal.startsWith(QLatin1String("__")) && !output.contains(signal))
             output.append(signal);
 
@@ -605,11 +628,13 @@ static QStringList getSortedSignalNameList(const ModelNode &modelNode)
     QStringList signalNames;
 
     if (metaInfo.isValid()) {
-        foreach (const PropertyName &signalName, sortedPropertyNameList(metaInfo.signalNames()))
+        const PropertyNameList signalNameList = sortedPropertyNameList(metaInfo.signalNames());
+        for (const PropertyName &signalName : signalNameList)
             if (!signalName.contains("Changed"))
                 signalNames.append(QString::fromUtf8(signalName));
 
-        foreach (const PropertyName &propertyName, sortedPropertyNameList(metaInfo.propertyNames()))
+        const PropertyNameList propertyNameList = sortedPropertyNameList(metaInfo.propertyNames());
+        for (const PropertyName &propertyName : propertyNameList)
             if (!propertyName.contains("."))
                 signalNames.append(QString::fromUtf8(propertyName + "Changed"));
     }
@@ -629,7 +654,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
     if (!qmlObjectNode.isValid()) {
         QString title = QCoreApplication::translate("ModelNodeOperations", "Go to Implementation");
-        QString description = QCoreApplication::translate("ModelNodeOperations", "Invalid item.");
+        QString description = QCoreApplication::translate("ModelNodeOperations", "Invalid component.");
         Core::AsynchronousMessageBox::warning(title, description);
         return;
     }
@@ -645,7 +670,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
     const Utils::FilePath currentDesignDocument = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()->fileName();
     const QString fileName = currentDesignDocument.toString();
-    const QString typeName = currentDesignDocument.toFileInfo().baseName();
+    const QString typeName = currentDesignDocument.baseName();
 
     QStringList signalNames = cleanSignalNames(getSortedSignalNameList(selectionState.selectedModelNodes().constFirst()));
 
@@ -663,7 +688,9 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
     Core::ModeManager::activateMode(Core::Constants::MODE_EDIT);
 
     if (!usages.isEmpty() && (addAlwaysNewSlot || usages.count() < 2)  && (!isModelNodeRoot  || addAlwaysNewSlot)) {
-        Core::EditorManager::openEditorAt(usages.constFirst().path, usages.constFirst().line, usages.constFirst().col);
+        Core::EditorManager::openEditorAt({Utils::FilePath::fromString(usages.constFirst().path),
+                                           usages.constFirst().line,
+                                           usages.constFirst().col});
 
         if (!signalNames.isEmpty()) {
             auto dialog = new AddSignalHandlerDialog(Core::ICore::dialogParent());
@@ -685,7 +712,9 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
                 //Move cursor to correct curser position
                 const QString filePath = Core::EditorManager::currentDocument()->filePath().toString();
                 QList<QmlJSEditor::FindReferences::Usage> usages = FindImplementation::run(filePath, typeName, itemId);
-                Core::EditorManager::openEditorAt(filePath, usages.constFirst().line, usages.constFirst().col + 1);
+                Core::EditorManager::openEditorAt({Utils::FilePath::fromString(filePath),
+                                                   usages.constFirst().line,
+                                                   usages.constFirst().col + 1});
             } );
             dialog->show();
 
@@ -693,7 +722,9 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
         return;
     }
 
-    Core::EditorManager::openEditorAt(usages.constFirst().path, usages.constFirst().line, usages.constFirst().col + 1);
+    Core::EditorManager::openEditorAt({Utils::FilePath::fromString(usages.constFirst().path),
+                                       usages.constFirst().line,
+                                       usages.constFirst().col + 1});
 }
 
 void removeLayout(const SelectionContext &selectionContext)
@@ -715,7 +746,8 @@ void removeLayout(const SelectionContext &selectionContext)
         return;
 
     selectionContext.view()->executeInTransaction("DesignerActionManager|removeLayout", [selectionContext, &layoutItem, parent](){
-        foreach (const ModelNode &modelNode, selectionContext.currentSingleSelectedNode().directSubModelNodes()) {
+        const QList<ModelNode> modelNodes = selectionContext.currentSingleSelectedNode().directSubModelNodes();
+        for (const ModelNode &modelNode : modelNodes) {
             if (QmlItemNode::isValidQmlItemNode(modelNode)) {
 
                 QmlItemNode qmlItem(modelNode);
@@ -761,6 +793,37 @@ void addNewSignalHandler(const SelectionContext &selectionState)
     addSignalHandlerOrGotoImplementation(selectionState, true);
 }
 
+// Open a model's material in the material editor
+void editMaterial(const SelectionContext &selectionContext)
+{
+    ModelNode modelNode = selectionContext.currentSingleSelectedNode();
+    QTC_ASSERT(modelNode.isValid(), return);
+
+    BindingProperty prop = modelNode.bindingProperty("materials");
+    if (!prop.exists())
+        return;
+
+    AbstractView *view = selectionContext.view();
+
+    ModelNode material;
+
+    if (view->hasId(prop.expression())) {
+        material = view->modelNodeForId(prop.expression());
+    } else {
+        QList<ModelNode> materials = prop.resolveToModelNodeList();
+
+        if (materials.size() > 0)
+            material = materials.first();
+    }
+
+    if (material.isValid()) {
+        QmlDesignerPlugin::instance()->mainWidget()->showDockWidget("MaterialEditor");
+
+        // to MaterialEditor and MaterialBrowser...
+        view->emitCustomNotification("selected_material_changed", {material});
+    }
+}
+
 void addItemToStackedContainer(const SelectionContext &selectionContext)
 {
     AbstractView *view = selectionContext.view();
@@ -961,83 +1024,100 @@ void addTabBarToStackedContainer(const SelectionContext &selectionContext)
 
 }
 
-Utils::FilePath projectFilePath()
-{
-    if (auto *doc = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()) {
-        if (auto *proj = ProjectExplorer::SessionManager::projectForFile(doc->fileName()))
-            return proj->projectDirectory();
-    }
-    return Utils::FilePath();
-}
-
-bool addFontToProject(const QStringList &fileNames, const QString &defaultDirectory)
-{
-
-    QString adjustedDefaultDirectory = defaultDirectory;
-    Utils::FilePath fonts = projectFilePath().pathAppended("fonts");
-
-    if (fonts.exists())
-        adjustedDefaultDirectory = fonts.toString();
-
-    QString directory = AddImagesDialog::getDirectory(fileNames, adjustedDefaultDirectory);
-
-    if (directory.isEmpty())
-        return true;
-
-    bool allSuccessful = true;
-    for (const QString &fileName : fileNames) {
-        const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
-        const bool success = QFile::copy(fileName, targetFile);
-
-        auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
-
-        QTC_ASSERT(document, return false);
-
-        if (success) {
-            ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
-            if (node) {
-                ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
-                if (containingFolder)
-                    containingFolder->addFiles(QStringList(targetFile));
-            }
-        } else {
-            allSuccessful = false;
-        }
-    }
-
-    return allSuccessful;
-}
-
-
-bool addImageToProject(const QStringList &fileNames, const QString &defaultDirectory)
+AddFilesResult addFilesToProject(const QStringList &fileNames, const QString &defaultDirectory)
 {
     QString directory = AddImagesDialog::getDirectory(fileNames, defaultDirectory);
-
     if (directory.isEmpty())
-        return true;
+        return AddFilesResult::Cancelled;
 
-    bool allSuccessful = true;
+    DesignDocument *document = QmlDesignerPlugin::instance()->currentDesignDocument();
+    QTC_ASSERT(document, return AddFilesResult::Failed);
+
+    QList<QPair<QString, QString>> copyList;
+    QStringList removeList;
     for (const QString &fileName : fileNames) {
         const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
-        const bool success = QFile::copy(fileName, targetFile);
-
-        auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
-
-        QTC_ASSERT(document, return false);
-
-        if (success) {
-            ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
-            if (node) {
-                ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
-                if (containingFolder)
-                    containingFolder->addFiles(QStringList(targetFile));
+        if (QFileInfo::exists(targetFile)) {
+            const QString title = QCoreApplication::translate(
+                        "ModelNodeOperations", "Overwrite Existing File?");
+            const QString question = QCoreApplication::translate(
+                        "ModelNodeOperations", "File already exists. Overwrite?\n\"%1\"").arg(targetFile);
+            if (QMessageBox::question(qobject_cast<QWidget *>(Core::ICore::dialogParent()),
+                                      title, question, QMessageBox::Yes | QMessageBox::No)
+                    != QMessageBox::Yes) {
+                continue;
             }
-        } else {
-            allSuccessful = false;
+            removeList.append(targetFile);
+        }
+        copyList.append({fileName, targetFile});
+    }
+    // Defer actual file operations after we have dealt with possible popup dialogs to avoid
+    // unnecessarily refreshing file models multiple times during the operation
+    for (const auto &file : qAsConst(removeList))
+        QFile::remove(file);
+
+    for (const auto &filePair : qAsConst(copyList)) {
+        const bool success = QFile::copy(filePair.first, filePair.second);
+        if (!success)
+            return AddFilesResult::Failed;
+
+        ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
+        if (node) {
+            ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
+            if (containingFolder)
+                containingFolder->addFiles({Utils::FilePath::fromString(filePair.second)});
         }
     }
 
-    return allSuccessful;
+    return AddFilesResult::Succeeded;
+}
+
+static QString getAssetDefaultDirectory(const QString &assetDir, const QString &defaultDirectory)
+{
+    QString adjustedDefaultDirectory = defaultDirectory;
+
+    Utils::FilePath contentPath = QmlDesignerPlugin::instance()->documentManager().currentProjectDirPath();
+
+    if (contentPath.pathAppended("content").exists())
+        contentPath = contentPath.pathAppended("content");
+
+    Utils::FilePath assetPath = contentPath.pathAppended(assetDir);
+
+    if (!assetPath.exists()) {
+        // Create the default asset type directory if it doesn't exist
+        QDir dir(contentPath.toString());
+        dir.mkpath(assetDir);
+    }
+
+    if (assetPath.exists() && assetPath.isDir())
+        adjustedDefaultDirectory = assetPath.toString();
+
+    return adjustedDefaultDirectory;
+}
+
+AddFilesResult addFontToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("fonts", defaultDirectory));
+}
+
+AddFilesResult addSoundToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("sounds", defaultDirectory));
+}
+
+AddFilesResult addShaderToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("shaders", defaultDirectory));
+}
+
+AddFilesResult addImageToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("images", defaultDirectory));
+}
+
+AddFilesResult addVideoToProject(const QStringList &fileNames, const QString &defaultDirectory)
+{
+    return addFilesToProject(fileNames, getAssetDefaultDirectory("videos", defaultDirectory));
 }
 
 void createFlowActionArea(const SelectionContext &selectionContext)
@@ -1435,7 +1515,7 @@ void styleMerge(const SelectionContext &selectionContext, const QString &templat
     QPlainTextEdit textEditTemplate;
     Utils::FileReader reader;
 
-    QTC_ASSERT(reader.fetch(templateFile), return);
+    QTC_ASSERT(reader.fetch(Utils::FilePath::fromString(templateFile)), return);
     QString qmlTemplateString = QString::fromUtf8(reader.data());
     QString imports;
 
@@ -1533,7 +1613,7 @@ void editAnnotation(const SelectionContext &selectionContext)
 {
     ModelNode selectedNode = selectionContext.currentSingleSelectedNode();
 
-    AnnotationEditor::showWidget(selectedNode);
+    ModelNodeEditorProxy::fromModelNode<AnnotationEditor>(selectedNode);
 }
 
 QVariant previewImageDataForGenericNode(const ModelNode &modelNode)
@@ -1550,6 +1630,23 @@ QVariant previewImageDataForImageNode(const ModelNode &modelNode)
     return {};
 }
 
+void openSignalDialog(const SelectionContext &selectionContext)
+{
+    if (!selectionContext.view() || !selectionContext.hasSingleSelectedModelNode())
+        return;
+
+    SignalList::showWidget(selectionContext.currentSingleSelectedNode());
+}
+
+void updateImported3DAsset(const SelectionContext &selectionContext)
+{
+    if (selectionContext.view()) {
+        selectionContext.view()->emitCustomNotification(
+                    "UpdateImported3DAsset", {selectionContext.currentSingleSelectedNode()});
+
+    }
+}
+
 } // namespace ModelNodeOperations
 
 } //QmlDesigner
