@@ -34,6 +34,7 @@
 #include "timelinepropertyitem.h"
 #include "timelinetoolbar.h"
 #include "timelineview.h"
+#include "navigation2d.h"
 
 #include <qmldesignerplugin.h>
 #include <qmlstate.h>
@@ -58,6 +59,10 @@
 #include <QSlider>
 #include <QVBoxLayout>
 #include <QtGlobal>
+#include <QSpacerItem>
+#include <QVariantAnimation>
+
+#include <cmath>
 
 namespace QmlDesigner {
 
@@ -118,12 +123,16 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     , m_timelineView(view)
     , m_graphicsScene(new TimelineGraphicsScene(this))
     , m_addButton(new QPushButton(this))
+    , m_onboardingContainer(new QWidget(this))
+    , m_loopPlayback(false)
+    , m_playbackSpeed(1)
+    , m_playbackAnimation(new QVariantAnimation(this))
 {
     setWindowTitle(tr("Timeline", "Title of timeline view"));
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     const QString css = Theme::replaceCssColors(QString::fromUtf8(
-        Utils::FileReader::fetchQrc(QLatin1String(":/qmldesigner/scrollbar.css"))));
+        Utils::FileReader::fetchQrc(":/qmldesigner/scrollbar.css")));
 
     m_scrollbar->setStyleSheet(css);
     m_scrollbar->setOrientation(Qt::Horizontal);
@@ -185,9 +194,71 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     m_addButton->setFlat(true);
     m_addButton->setFixedSize(32, 32);
 
+
+    widgetLayout->addWidget(m_onboardingContainer);
+
+    auto *onboardingTopLabel = new QLabel(m_onboardingContainer);
+    auto *onboardingBottomLabel = new QLabel(m_onboardingContainer);
+    auto *onboardingBottomIcon = new QLabel(m_onboardingContainer);
+
+    auto *onboardingLayout = new QVBoxLayout;
+    auto *onboardingSublayout = new QHBoxLayout;
+    auto *leftSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
+    auto *rightSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
+    auto *topSpacer = new QSpacerItem(40, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);
+    auto *bottomSpacer = new QSpacerItem(40, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);
+
+    const QString labelText =
+            tr("This file does not contain a timeline. <br><br>"
+               "To create an animation, add a timeline by clicking the + button.");
+
+    onboardingTopLabel->setText(labelText);
+    onboardingTopLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
+
+    m_onboardingContainer->setLayout(onboardingLayout);
+    onboardingLayout->setContentsMargins(0, 0, 0, 0);
+    onboardingLayout->setSpacing(0);
+    onboardingLayout->addSpacerItem(topSpacer);
+    onboardingLayout->addWidget(onboardingTopLabel);
+    onboardingLayout->addLayout(onboardingSublayout);
+
+    onboardingSublayout->setContentsMargins(0, 0, 0, 0);
+    onboardingSublayout->setSpacing(0);
+    onboardingSublayout->addSpacerItem(leftSpacer);
+
+    onboardingBottomLabel->setAlignment(Qt::AlignRight | Qt::AlignTop);
+    onboardingBottomLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    onboardingSublayout->addWidget(onboardingBottomLabel);
+    onboardingBottomLabel->setText(tr("To edit the timeline settings, click "));
+
+    onboardingBottomIcon->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+    onboardingBottomIcon->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    onboardingSublayout->addWidget(onboardingBottomIcon);
+    onboardingBottomIcon->setPixmap(TimelineIcons::ANIMATION.pixmap());
+
+    onboardingSublayout->addSpacerItem(rightSpacer);
+    onboardingLayout->addSpacerItem(bottomSpacer);
+
     widgetLayout->addLayout(contentLayout);
     this->setLayout(widgetLayout);
 
+    {
+        QPalette timelinePalette;
+        timelinePalette.setColor(QPalette::Text, Utils::creatorTheme()->color(
+                                     Utils::Theme::DStextColor));
+        timelinePalette.setColor(QPalette::WindowText, timelinePalette.color(QPalette::Text));
+        timelinePalette.setColor(QPalette::Window, Utils::creatorTheme()->color(
+                                     Utils::Theme::QmlDesigner_BackgroundColorDarkAlternate));
+
+        onboardingTopLabel->setPalette(timelinePalette);
+        onboardingBottomLabel->setPalette(timelinePalette);
+        setPalette(timelinePalette);
+        setAutoFillBackground(true);
+        m_graphicsView->setPalette(timelinePalette);
+        m_graphicsView->setBackgroundRole(QPalette::Window);
+        m_statusBar->setPalette(timelinePalette);
+    }
+
     connectToolbar();
 
     auto setScrollOffset = [this]() { graphicsScene()->setScrollOffset(m_scrollbar->value()); };
@@ -201,6 +272,38 @@ TimelineWidget::TimelineWidget(TimelineView *view)
     connect(m_addButton, &QPushButton::clicked, this, [this]() {
         m_timelineView->addNewTimelineDialog();
     });
+
+    Navigation2dFilter *filter = new Navigation2dFilter(m_graphicsView->viewport());
+    connect(filter, &Navigation2dFilter::panChanged, [this](const QPointF &direction) {
+        Navigation2dFilter::scroll(direction, m_scrollbar, m_graphicsView->verticalScrollBar());
+    });
+
+    connect(filter, &Navigation2dFilter::zoomChanged, [this](double scale, const QPointF &pos) {
+        int s = static_cast<int>(std::round(scale*100.));
+        int scaleFactor = std::clamp(m_graphicsScene->zoom() + s, 0, 100);
+        double ps = m_graphicsScene->mapFromScene(pos.x());
+        m_graphicsScene->setZoom(scaleFactor, ps);
+        m_toolbar->setScaleFactor(scaleFactor);
+    });
+    m_graphicsView->viewport()->installEventFilter(filter);
+
+    m_playbackAnimation->stop();
+    auto playAnimation = [this](QVariant frame) { graphicsScene()->setCurrentFrame(qRound(frame.toDouble())); };
+    connect(m_playbackAnimation, &QVariantAnimation::valueChanged, playAnimation);
+
+    auto updatePlaybackLoopValues = [this]() {
+        updatePlaybackValues();
+    };
+    connect(graphicsScene()->layoutRuler(), &TimelineRulerSectionItem::playbackLoopValuesChanged, updatePlaybackLoopValues);
+
+    auto setPlaybackState = [this](QAbstractAnimation::State newState, QAbstractAnimation::State oldState) {
+        Q_UNUSED(oldState)
+        m_toolbar->setPlayState(newState == QAbstractAnimation::State::Running);
+    };
+    connect(m_playbackAnimation, &QVariantAnimation::stateChanged, setPlaybackState);
+
+    auto onFinish = [this]() { graphicsScene()->setCurrentFrame(m_playbackAnimation->startValue().toInt()); };
+    connect(m_playbackAnimation, &QVariantAnimation::finished, onFinish);
 }
 
 void TimelineWidget::connectToolbar()
@@ -212,10 +315,8 @@ void TimelineWidget::connectToolbar()
 
     connect(graphicsScene(), &TimelineGraphicsScene::scroll, this, &TimelineWidget::scroll);
 
-    connect(m_toolbar, &TimelineToolBar::curveChanged, this, &TimelineWidget::updateAnimationCurve);
-
-    auto setRulerScaling = [this](int val) { m_graphicsScene->setRulerScaling(val); };
-    connect(m_toolbar, &TimelineToolBar::scaleFactorChanged, setRulerScaling);
+    auto setZoomFactor = [this](int val) { m_graphicsScene->setZoom(val); };
+    connect(m_toolbar, &TimelineToolBar::scaleFactorChanged, setZoomFactor);
 
     auto setToFirstFrame = [this]() {
         graphicsScene()->setCurrentFrame(graphicsScene()->startFrame());
@@ -246,6 +347,21 @@ void TimelineWidget::connectToolbar()
 
     connect(m_toolbar, &TimelineToolBar::recordToggled, this, &TimelineWidget::setTimelineRecording);
 
+    connect(m_toolbar, &TimelineToolBar::playTriggered, this, &TimelineWidget::toggleAnimationPlayback);
+
+    auto setLoopAnimation = [this](bool loop) {
+        graphicsScene()->layoutRuler()->setPlaybackLoopEnabled(loop);
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running)
+            m_playbackAnimation->pause();
+        m_loopPlayback = loop;
+    };
+    connect(m_toolbar, &TimelineToolBar::loopPlaybackToggled, setLoopAnimation);
+    auto setPlaybackSpeed = [this](float val) {
+        m_playbackSpeed = val;
+        updatePlaybackValues();
+    };
+    connect(m_toolbar, &TimelineToolBar::playbackSpeedChanged, setPlaybackSpeed);
+
     connect(m_toolbar,
             &TimelineToolBar::openEasingCurveEditor,
             this,
@@ -255,19 +371,6 @@ void TimelineWidget::connectToolbar()
             &TimelineToolBar::settingDialogClicked,
             m_timelineView,
             &TimelineView::openSettingsDialog);
-
-    for (auto action : QmlDesignerPlugin::instance()->designerActionManager().designerActions()) {
-        if (action->menuId() == "LivePreview") {
-            QObject::connect(m_toolbar,
-                             &TimelineToolBar::playTriggered,
-                             action->action(),
-                             [action]() {
-                                 action->action()->setChecked(false);
-                                 action->action()->triggered(true);
-                             });
-        }
-    }
-
     setTimelineActive(false);
 }
 
@@ -291,101 +394,6 @@ void TimelineWidget::scroll(const TimelineUtils::Side &side)
         m_scrollbar->setValue(m_scrollbar->value() + m_scrollbar->singleStep());
 }
 
-ModelNode getTargetNode(DesignTools::PropertyTreeItem *item, const QmlTimeline &timeline)
-{
-    if (const DesignTools::NodeTreeItem *nodeItem = item->parentNodeTreeItem()) {
-        QString targetId = nodeItem->name();
-        if (timeline.isValid()) {
-            for (auto &&target : timeline.allTargets()) {
-                if (target.displayName() == targetId)
-                    return target;
-            }
-        }
-    }
-    return ModelNode();
-}
-
-QmlTimelineKeyframeGroup timelineKeyframeGroup(QmlTimeline &timeline,
-                                               DesignTools::PropertyTreeItem *item)
-{
-    ModelNode node = getTargetNode(item, timeline);
-    if (node.isValid())
-        return timeline.keyframeGroup(node, item->name().toLatin1());
-
-    return QmlTimelineKeyframeGroup();
-}
-
-void attachEasingCurve(double frame,
-                       const QEasingCurve &curve,
-                       const QmlTimelineKeyframeGroup &group)
-{
-    ModelNode frameNode = group.keyframe(frame);
-    if (frameNode.isValid()) {
-        auto expression = EasingCurve(curve).toString();
-        frameNode.bindingProperty("easing.bezierCurve").setExpression(expression);
-    }
-}
-
-void TimelineWidget::updateAnimationCurve(DesignTools::PropertyTreeItem *item)
-{
-    QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
-    QmlTimelineKeyframeGroup group = timelineKeyframeGroup(currentTimeline, item);
-
-    if (group.isValid()) {
-        ModelNode groupNode = group.modelNode();
-
-        if (groupNode.isValid()) {
-            if (item->locked())
-                groupNode.setAuxiliaryData("locked", true);
-            else
-                groupNode.removeAuxiliaryData("locked");
-
-            if (item->pinned())
-                groupNode.setAuxiliaryData("pinned", true);
-            else
-                groupNode.removeAuxiliaryData("pinned");
-
-            if (item->hasUnified())
-                groupNode.setAuxiliaryData("unified", item->unifyString());
-            else
-                groupNode.removeAuxiliaryData("unified");
-        }
-
-        auto replaceKeyframes = [&group, item, this]() {
-            m_toolbar->setBlockReflection(true);
-            for (auto frame : group.keyframes())
-                frame.destroy();
-
-            DesignTools::Keyframe previous;
-            for (auto &&frame : item->curve().keyframes()) {
-                QPointF pos = frame.position();
-                group.setValue(QVariant(pos.y()), pos.x());
-
-                if (previous.isValid()) {
-                    if (frame.interpolation() == DesignTools::Keyframe::Interpolation::Bezier) {
-                        DesignTools::CurveSegment segment(previous, frame);
-                        if (segment.isValid())
-                            attachEasingCurve(pos.x(), segment.easingCurve(), group);
-                    } else if (frame.interpolation()
-                               == DesignTools::Keyframe::Interpolation::Easing) {
-                        QVariant data = frame.data();
-                        if (data.type() == static_cast<int>(QMetaType::QEasingCurve))
-                            attachEasingCurve(pos.x(), data.value<QEasingCurve>(), group);
-                    } else if (frame.interpolation() == DesignTools::Keyframe::Interpolation::Step) {
-                        // Warning: Keyframe::Interpolation::Step not yet implemented
-                    }
-                }
-
-                previous = frame;
-            }
-            m_toolbar->setBlockReflection(false);
-        };
-
-        timelineView()->executeInTransaction("TimelineWidget::handleKeyframeReplacement",
-                                             replaceKeyframes);
-    }
-}
-
 void TimelineWidget::selectionChanged()
 {
     if (graphicsScene()->hasSelection())
@@ -404,6 +412,54 @@ void TimelineWidget::openEasingCurveEditor()
     }
 }
 
+void TimelineWidget::updatePlaybackValues()
+{
+    QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+    qreal endFrame = currentTimeline.endKeyframe();
+    qreal startFrame = currentTimeline.startKeyframe();
+    qreal duration = currentTimeline.duration();
+
+    if (m_loopPlayback) {
+        m_playbackAnimation->setLoopCount(-1);
+        qreal loopStart = graphicsScene()->layoutRuler()->playbackLoopStart();
+        qreal loopEnd = graphicsScene()->layoutRuler()->playbackLoopEnd();
+        startFrame = qRound(startFrame + loopStart);
+        endFrame = qRound(startFrame + (loopEnd - loopStart));
+        duration = endFrame - startFrame;
+    } else {
+        m_playbackAnimation->setLoopCount(1);
+    }
+
+    if (duration > 0.0f) {
+        qreal animationDuration = duration * (1.0 / m_playbackSpeed);
+        m_playbackAnimation->setDuration(animationDuration);
+    } else {
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running)
+            m_playbackAnimation->stop();
+    }
+    qreal newCurrentTime = (currentTimeline.currentKeyframe() - startFrame) * (1.0 / m_playbackSpeed);
+    if (qRound(m_playbackAnimation->startValue().toDouble()) != qRound(startFrame)
+        || qRound(m_playbackAnimation->endValue().toDouble()) != qRound(endFrame)) {
+        newCurrentTime = 0;
+    }
+    m_playbackAnimation->setStartValue(qRound(startFrame));
+    m_playbackAnimation->setEndValue(qRound(endFrame));
+    m_playbackAnimation->setCurrentTime(newCurrentTime);
+}
+
+void TimelineWidget::toggleAnimationPlayback()
+{
+    QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+    if (currentTimeline.isValid() && m_playbackSpeed > 0.0) {
+        if (m_playbackAnimation->state() == QAbstractAnimation::Running) {
+            m_playbackAnimation->pause();
+        } else {
+            updatePlaybackValues();
+            m_playbackAnimation->start();
+        }
+    }
+}
+
 void TimelineWidget::setTimelineRecording(bool value)
 {
     ModelNode node = timelineView()->modelNodeForId(m_toolbar->currentTimelineId());
@@ -412,7 +468,7 @@ void TimelineWidget::setTimelineRecording(bool value)
         timelineView()->activateTimelineRecording(node);
     } else {
         timelineView()->deactivateTimelineRecording();
-        timelineView()->activateTimeline(node);
+        timelineView()->setCurrentTimeline(node);
     }
 
     graphicsScene()->invalidateRecordButtonsStatus();
@@ -426,7 +482,7 @@ void TimelineWidget::contextHelp(const Core::IContext::HelpCallback &callback) c
         callback({});
 }
 
-void TimelineWidget::init()
+void TimelineWidget::init(int zoom)
 {
     QmlTimeline currentTimeline = m_timelineView->timelineForState(m_timelineView->currentState());
     if (currentTimeline.isValid()) {
@@ -438,13 +494,13 @@ void TimelineWidget::init()
         m_statusBar->clear();
     }
 
-    invalidateTimelineDuration(m_graphicsScene->currentTimeline());
+    invalidateTimelineDuration(currentTimeline);
 
     m_graphicsScene->setWidth(m_graphicsView->viewport()->width());
 
     // setScaleFactor uses QSignalBlocker.
-    m_toolbar->setScaleFactor(0);
-    m_graphicsScene->setRulerScaling(0);
+    m_toolbar->setScaleFactor(zoom);
+    m_graphicsScene->setZoom(zoom);
 }
 
 void TimelineWidget::reset()
@@ -467,7 +523,7 @@ TimelineToolBar *TimelineWidget::toolBar() const
 void TimelineWidget::invalidateTimelineDuration(const QmlTimeline &timeline)
 {
     if (timelineView() && timelineView()->model()) {
-        QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+        QmlTimeline currentTimeline = timelineView()->currentTimeline();
         if (currentTimeline.isValid() && currentTimeline == timeline) {
             m_toolbar->setStartFrame(timeline.startKeyframe());
             m_toolbar->setEndFrame(timeline.endKeyframe());
@@ -491,7 +547,7 @@ void TimelineWidget::invalidateTimelineDuration(const QmlTimeline &timeline)
 void TimelineWidget::invalidateTimelinePosition(const QmlTimeline &timeline)
 {
     if (timelineView() && timelineView()->model()) {
-        QmlTimeline currentTimeline = graphicsScene()->currentTimeline();
+        QmlTimeline currentTimeline = timelineView()->currentTimeline();
         if (currentTimeline.isValid() && currentTimeline == timeline) {
             qreal frame = getcurrentFrame(timeline);
             m_toolbar->setCurrentFrame(frame);
@@ -512,16 +568,25 @@ void TimelineWidget::setupScrollbar(int min, int max, int current)
 
 void TimelineWidget::setTimelineId(const QString &id)
 {
+    auto currentState = m_timelineView->currentState();
+    auto timelineOfState = m_timelineView->timelineForState(currentState.modelNode());
+
+    bool active = false;
+    if (auto node = timelineOfState.modelNode(); node.isValid())
+        active = node.id() == id;
+
     const bool empty = m_timelineView->getTimelines().isEmpty();
-    setTimelineActive(!empty);
-    if (m_timelineView->isAttached() && !empty) {
-        m_toolbar->setCurrentTimeline(m_timelineView->modelNodeForId(id));
-        m_toolbar->setCurrentState(m_timelineView->currentState().name());
-        m_timelineView->setTimelineRecording(false);
+    const bool valid = active && !empty;
+
+    setTimelineActive(valid);
+    if (m_timelineView->isAttached() && valid) {
+        m_toolbar->setCurrentTimeline(timelineOfState);
+        m_toolbar->setCurrentState(currentState.name());
     } else {
         m_toolbar->setCurrentTimeline({});
         m_toolbar->setCurrentState({});
     }
+    m_timelineView->setTimelineRecording(false);
 }
 
 void TimelineWidget::setTimelineActive(bool b)
@@ -532,6 +597,7 @@ void TimelineWidget::setTimelineActive(bool b)
         m_rulerView->setVisible(true);
         m_scrollbar->setVisible(true);
         m_addButton->setVisible(false);
+        m_onboardingContainer->setVisible(false);
         m_graphicsView->update();
         m_rulerView->update();
     } else {
@@ -539,17 +605,36 @@ void TimelineWidget::setTimelineActive(bool b)
         m_graphicsView->setVisible(false);
         m_rulerView->setVisible(false);
         m_scrollbar->setVisible(false);
+        m_statusBar->clear();
         m_addButton->setVisible(true);
+        m_onboardingContainer->setVisible(true);
     }
 }
 
+void TimelineWidget::setFocus()
+{
+    m_graphicsView->setFocus();
+}
+
 void TimelineWidget::showEvent(QShowEvent *event)
 {
     Q_UNUSED(event)
+
+    int zoom = m_toolbar->scaleFactor();
+
+    m_timelineView->setEnabled(true);
+
     graphicsScene()->setWidth(m_graphicsView->viewport()->width());
+    graphicsScene()->invalidateScene();
     graphicsScene()->invalidateLayout();
     graphicsScene()->invalidate();
     graphicsScene()->onShow();
+
+    QWidget::showEvent(event);
+
+    //All the events have to be fully processed before we call init()
+    if (m_timelineView->model())
+        QTimer::singleShot(0, [this, zoom]() { init(zoom); });
 }
 
 void TimelineWidget::resizeEvent(QResizeEvent *event)
@@ -558,6 +643,12 @@ void TimelineWidget::resizeEvent(QResizeEvent *event)
     graphicsScene()->setWidth(m_graphicsView->viewport()->width());
 }
 
+void TimelineWidget::hideEvent(QHideEvent *event)
+{
+    m_timelineView->setEnabled(false);
+    QWidget::hideEvent(event);
+}
+
 TimelineView *TimelineWidget::timelineView() const
 {
     return m_timelineView;
