@@ -44,6 +44,7 @@
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectmanager.h>
 
+#include <qmlprojectmanager/projectfilecontenttools.h>
 #include <qmlprojectmanager/qmlproject.h>
 
 #include <qmldesigner/components/componentcore/theme.h>
@@ -54,7 +55,6 @@
 #include <utils/hostosinfo.h>
 #include <utils/icon.h>
 #include <utils/infobar.h>
-#include <utils/mimetypes/mimedatabase.h>
 #include <utils/stringutils.h>
 #include <utils/theme/theme.h>
 
@@ -213,12 +213,16 @@ public:
 
     Q_INVOKABLE void createProject()
     {
-        ProjectExplorer::ProjectExplorerPlugin::openNewProjectDialog();
+        QTimer::singleShot(0, []() {
+            ProjectExplorer::ProjectExplorerPlugin::openNewProjectDialog();
+        });
     }
 
     Q_INVOKABLE void openProject()
     {
-        ProjectExplorer::ProjectExplorerPlugin::openOpenProjectDialog();
+        QTimer::singleShot(0, []() {
+            ProjectExplorer::ProjectExplorerPlugin::openOpenProjectDialog();
+        });
     }
 
     Q_INVOKABLE void openProjectAt(int row)
@@ -326,43 +330,10 @@ int ProjectModel::rowCount(const QModelIndex &) const
 
 QString getQDSVersion(const QString &projectFilePath)
 {
-    const QString defaultReturn = "";
-    Utils::FileReader reader;
-    if (!reader.fetch(Utils::FilePath::fromString(projectFilePath)))
-        return defaultReturn;
+    const QString qdsVersion = QmlProjectManager::ProjectFileContentTools::qdsVersion(
+                                        Utils::FilePath::fromString(projectFilePath));
 
-    const QByteArray data = reader.data();
-
-    QRegularExpression regexp(R"x(qdsVersion: "(.*)")x");
-    QRegularExpressionMatch match = regexp.match(QString::fromUtf8(data));
-
-    if (!match.hasMatch())
-        return defaultReturn;
-
-    return ProjectModel::tr("Created with Qt Design Studio version: %1").arg(match.captured(1));
-}
-
-QString getMainQmlFile(const QString &projectFilePath)
-{
-    const QString defaultReturn = "content/App.qml";
-    Utils::FileReader reader;
-    if (!reader.fetch(Utils::FilePath::fromString(projectFilePath)))
-            return defaultReturn;
-
-    const QByteArray data = reader.data();
-
-    QRegularExpression regexp(R"x(mainFile: "(.*)")x");
-    QRegularExpressionMatch match = regexp.match(QString::fromUtf8(data));
-
-    if (!match.hasMatch())
-        return defaultReturn;
-
-    return match.captured(1);
-}
-
-QString appQmlFile(const QString &projectFilePath)
-{
-    return QFileInfo(projectFilePath).dir().absolutePath() + "/" +  getMainQmlFile(projectFilePath);
+    return ProjectModel::tr("Created with Qt Design Studio version: %1").arg(qdsVersion);
 }
 
 static QString fromCamelCase(const QString &s) {
@@ -378,32 +349,12 @@ static QString fromCamelCase(const QString &s) {
 
 static QString resolutionFromConstants(const QString &projectFilePath)
 {
-    const QFileInfo fileInfo(projectFilePath);
-    const QString fileName = fileInfo.dir().absolutePath()
-            + "/"  + "imports" + "/" + fileInfo.baseName() + "/Constants.qml";
+    QmlProjectManager::ProjectFileContentTools::Resolution res =
+            QmlProjectManager::ProjectFileContentTools::resolutionFromConstants(
+                Utils::FilePath::fromString(projectFilePath));
 
-    Utils::FileReader reader;
-    if (!reader.fetch(Utils::FilePath::fromString(fileName)))
-        return {};
-
-    const QByteArray data = reader.data();
-
-    const QRegularExpression regexpWidth(R"x(readonly\s+property\s+int\s+width:\s+(\d*))x");
-    const QRegularExpression regexpHeight(R"x(readonly\s+property\s+int\s+height:\s+(\d*))x");
-
-    int width = -1;
-    int height = -1;
-
-    QRegularExpressionMatch match = regexpHeight.match(QString::fromUtf8(data));
-    if (match.hasMatch())
-        height = match.captured(1).toInt();
-
-    match = regexpWidth.match(QString::fromUtf8(data));
-    if (match.hasMatch())
-        width = match.captured(1).toInt();
-
-    if (width > 0 && height > 0)
-        return ProjectModel::tr("Resolution: %1x%2").arg(width).arg(height);
+    if (res.width > 0 && res.height > 0)
+        return ProjectModel::tr("Resolution: %1x%2").arg(res.width).arg(res.height);
 
     return {};
 }
@@ -445,8 +396,8 @@ static QString tags(const QString &projectFilePath)
 
 QVariant ProjectModel::data(const QModelIndex &index, int role) const
 {
-    QPair<QString, QString> data = ProjectExplorer::ProjectExplorerPlugin::recentProjects().at(
-        index.row());
+    const ProjectExplorer::RecentProjectsEntry data =
+            ProjectExplorer::ProjectExplorerPlugin::recentProjects().at(index.row());
     switch (role) {
     case Qt::DisplayRole:
         return data.second;
@@ -456,7 +407,9 @@ QVariant ProjectModel::data(const QModelIndex &index, int role) const
     case PrettyFilePathRole:
         return Utils::withTildeHomePath(QFileInfo(data.first).dir().absolutePath());
     case PreviewUrl:
-        return QVariant(QStringLiteral("image://project_preview/") + appQmlFile(data.first));
+        return QVariant(QStringLiteral("image://project_preview/") +
+                        QmlProjectManager::ProjectFileContentTools::appQmlFile(
+                            Utils::FilePath::fromString(data.first)));
     case TagData:
         return tags(data.first);
     case Description:
@@ -584,7 +537,7 @@ void StudioWelcomePlugin::extensionsInitialized()
 
         const QString filters = QString("Project (*.qmlproject);;UI file (*.ui.qml);;QML file "
                                         "(*.qml);;JavaScript file (*.js);;%1")
-                                    .arg(Utils::allFilesFilterString());
+                                    .arg(Core::DocumentManager::allFilesFilterString());
 
         Core::DocumentManager::setFileDialogFilter(filters);
     }
@@ -707,10 +660,12 @@ WelcomeMode::WelcomeMode()
 
     m_dataModelDownloader->start();
 
+/*
     connect(Core::ModeManager::instance(), &Core::ModeManager::currentModeChanged, this, [this](Utils::Id mode){
        bool active = (mode == Core::Constants::MODE_WELCOME);
        m_modeWidget->rootObject()->setProperty("active", active);
     });
+*/
 
     if (!useNewWelcomePage()) {
 
@@ -733,9 +688,9 @@ WelcomeMode::WelcomeMode()
 
         QShortcut *updateShortcut = nullptr;
         if (Utils::HostOsInfo::isMacHost())
-            updateShortcut = new QShortcut(QKeySequence(Qt::ALT + Qt::Key_F5), m_modeWidget);
+            updateShortcut = new QShortcut(QKeySequence(Qt::ALT | Qt::Key_F5), m_modeWidget);
         else
-            updateShortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_F5), m_modeWidget);
+            updateShortcut = new QShortcut(QKeySequence(Qt::CTRL | Qt::Key_F5), m_modeWidget);
         connect(updateShortcut, &QShortcut::activated, this, [this, welcomePagePath](){
             m_modeWidget->setSource(QUrl::fromLocalFile(welcomePagePath + "/main.qml"));
         });
