@@ -26,8 +26,8 @@
 #include <languageserverprotocol/basemessage.h>
 #include <languageserverprotocol/jsonobject.h>
 #include <languageserverprotocol/jsonrpcmessages.h>
-#include <utils/mimetypes/mimetype.h>
-#include <utils/mimetypes/mimedatabase.h>
+
+#include <utils/hostosinfo.h>
 
 #include <QTextCodec>
 #include <QtTest>
@@ -36,8 +36,8 @@ using namespace LanguageServerProtocol;
 
 Q_DECLARE_METATYPE(QTextCodec *)
 Q_DECLARE_METATYPE(BaseMessage)
-Q_DECLARE_METATYPE(JsonRpcMessage)
 Q_DECLARE_METATYPE(DocumentUri)
+Q_DECLARE_METATYPE(Range)
 
 class tst_LanguageServerProtocol : public QObject
 {
@@ -60,11 +60,12 @@ private slots:
     void jsonMessageToBaseMessage_data();
     void jsonMessageToBaseMessage();
 
-    void jsonObject();
-
     void documentUri_data();
     void documentUri();
 
+    void range_data();
+    void range();
+
 private:
     QByteArray defaultMimeType;
     QTextCodec *defaultCodec = nullptr;
@@ -72,7 +73,7 @@ private:
 
 void tst_LanguageServerProtocol::initTestCase()
 {
-    defaultMimeType = JsonRpcMessageHandler::jsonRpcMimeType();
+    defaultMimeType = JsonRpcMessage::jsonRpcMimeType();
     defaultCodec = QTextCodec::codecForName("utf-8");
 }
 
@@ -306,7 +307,7 @@ void tst_LanguageServerProtocol::baseMessageParse()
     BaseMessage::parse(&buffer, parseError, partial);
 
     if (!parseError.isEmpty() && !error) // show message if there is an error message we do not expect
-        QWARN(parseError.toLatin1());
+        qWarning() << qPrintable(parseError);
     QCOMPARE(!parseError.isEmpty(), error);
     QCOMPARE(partial.content, content);
     QCOMPARE(partial.isValid(), valid);
@@ -358,7 +359,7 @@ void tst_LanguageServerProtocol::baseMessageToData()
     QFETCH(BaseMessage, message);
     QFETCH(QByteArray, data);
 
-    QCOMPARE(message.toData(), data);
+    QCOMPARE(message.header() + message.content, data);
 }
 
 void tst_LanguageServerProtocol::fromJsonValue()
@@ -447,13 +448,13 @@ void tst_LanguageServerProtocol::toJsonObject()
     QFETCH(bool, error);
     QFETCH(QJsonObject, expected);
 
-    QString parseError;
-    const QJsonObject object = JsonRpcMessageHandler::toJsonObject(content, codec, parseError);
+    BaseMessage baseMessage(JsonRpcMessage::jsonRpcMimeType(), content, content.length(), codec);
+    JsonRpcMessage jsonRpcMessage(baseMessage);
 
-    if (!error && !parseError.isEmpty())
-        QFAIL(parseError.toLocal8Bit().data());
-    QCOMPARE(object, expected);
-    QCOMPARE(!parseError.isEmpty(), error);
+    if (!error && !jsonRpcMessage.parseError().isEmpty())
+        QFAIL(jsonRpcMessage.parseError().toLocal8Bit().data());
+    QCOMPARE(jsonRpcMessage.toJsonObject(), expected);
+    QCOMPARE(!jsonRpcMessage.parseError().isEmpty(), error);
 }
 
 void tst_LanguageServerProtocol::jsonMessageToBaseMessage_data()
@@ -462,11 +463,11 @@ void tst_LanguageServerProtocol::jsonMessageToBaseMessage_data()
     QTest::addColumn<BaseMessage>("baseMessage");
 
     QTest::newRow("empty object") << JsonRpcMessage(QJsonObject())
-                                  << BaseMessage(JsonRpcMessageHandler::jsonRpcMimeType(),
+                                  << BaseMessage(JsonRpcMessage::jsonRpcMimeType(),
                                                  "{}");
 
-    QTest::newRow("key value pair") << JsonRpcMessage({{"key", "value"}})
-                                    << BaseMessage(JsonRpcMessageHandler::jsonRpcMimeType(),
+    QTest::newRow("key value pair") << JsonRpcMessage(QJsonObject{{"key", "value"}})
+                                    << BaseMessage(JsonRpcMessage::jsonRpcMimeType(),
                                                    R"({"key":"value"})");
 }
 
@@ -478,126 +479,6 @@ void tst_LanguageServerProtocol::jsonMessageToBaseMessage()
     QCOMPARE(jsonMessage.toBaseMessage(), baseMessage);
 }
 
-class JsonTestObject : public JsonObject
-{
-public:
-    using JsonObject::JsonObject;
-    using JsonObject::insert;
-    using JsonObject::value;
-    using JsonObject::contains;
-    using JsonObject::find;
-    using JsonObject::end;
-    using JsonObject::remove;
-    using JsonObject::keys;
-    using JsonObject::typedValue;
-    using JsonObject::optionalValue;
-    using JsonObject::clientValue;
-    using JsonObject::optionalClientValue;
-    using JsonObject::array;
-    using JsonObject::optionalArray;
-    using JsonObject::clientArray;
-    using JsonObject::optionalClientArray;
-    using JsonObject::insertArray;
-    using JsonObject::checkKey;
-    using JsonObject::valueTypeString;
-    using JsonObject::check;
-    using JsonObject::checkType;
-    using JsonObject::checkVal;
-    using JsonObject::checkArray;
-    using JsonObject::checkOptional;
-    using JsonObject::checkOptionalArray;
-    using JsonObject::errorString;
-    using JsonObject::operator==;
-};
-
-void tst_LanguageServerProtocol::jsonObject()
-{
-    JsonTestObject obj;
-
-    obj.insert("integer", 42);
-    obj.insert("double", 42.42);
-    obj.insert("bool", false);
-    obj.insert("null", QJsonValue::Null);
-    obj.insert("string", "foobar");
-    obj.insertArray("strings", QStringList{"foo", "bar"});
-    const JsonTestObject innerObj(obj);
-    obj.insert("object", innerObj);
-
-    QCOMPARE(obj.value("integer"), QJsonValue(42));
-    QCOMPARE(obj.value("double"), QJsonValue(42.42));
-    QCOMPARE(obj.value("bool"), QJsonValue(false));
-    QCOMPARE(obj.value("null"), QJsonValue(QJsonValue::Null));
-    QCOMPARE(obj.value("string"), QJsonValue("foobar"));
-    QCOMPARE(obj.value("strings"), QJsonValue(QJsonArray({"foo", "bar"})));
-    QCOMPARE(obj.value("object"), QJsonValue(QJsonObject(innerObj)));
-
-    QCOMPARE(obj.typedValue<int>("integer"), 42);
-    QCOMPARE(obj.typedValue<double>("double"), 42.42);
-    QCOMPARE(obj.typedValue<bool>("bool"), false);
-    QCOMPARE(obj.typedValue<QString>("string"), QString("foobar"));
-    QCOMPARE(obj.typedValue<JsonTestObject>("object"), innerObj);
-
-    QVERIFY(!obj.optionalValue<int>("doesNotExist").has_value());
-    QVERIFY(obj.optionalValue<int>("integer").has_value());
-    QCOMPARE(obj.optionalValue<int>("integer").value_or(0), 42);
-
-    QVERIFY(obj.clientValue<int>("null").isNull());
-    QVERIFY(!obj.clientValue<int>("integer").isNull());
-    QCOMPARE(obj.clientValue<int>("integer").value(), 42);
-
-    QVERIFY(!obj.optionalClientValue<int>("doesNotExist").has_value());
-    QVERIFY(obj.optionalClientValue<int>("null").has_value());
-    QVERIFY(obj.optionalClientValue<int>("null").value().isNull());
-    QVERIFY(obj.optionalClientValue<int>("integer").has_value());
-    QVERIFY(!obj.optionalClientValue<int>("integer").value().isNull());
-    QCOMPARE(obj.optionalClientValue<int>("integer").value().value(0), 42);
-
-    QCOMPARE(obj.array<QString>("strings"), QList<QString>({"foo", "bar"}));
-
-    QVERIFY(!obj.optionalArray<QString>("doesNotExist").has_value());
-    QVERIFY(obj.optionalArray<QString>("strings").has_value());
-    QCOMPARE(obj.optionalArray<QString>("strings").value_or(QList<QString>()),
-             QList<QString>({"foo", "bar"}));
-
-    QVERIFY(obj.clientArray<QString>("null").isNull());
-    QVERIFY(!obj.clientArray<QString>("strings").isNull());
-    QCOMPARE(obj.clientArray<QString>("strings").toList(), QList<QString>({"foo", "bar"}));
-
-    QVERIFY(!obj.optionalClientArray<QString>("doesNotExist").has_value());
-    QVERIFY(obj.optionalClientArray<QString>("null").has_value());
-    QVERIFY(obj.optionalClientArray<QString>("null").value().isNull());
-    QVERIFY(obj.optionalClientArray<QString>("strings").has_value());
-    QVERIFY(!obj.optionalClientArray<QString>("strings").value().isNull());
-    QCOMPARE(obj.optionalClientArray<QString>("strings").value().toList(),
-             QList<QString>({"foo", "bar"}));
-
-    ErrorHierarchy errorHierarchy;
-    QVERIFY(!obj.check<int>(&errorHierarchy, "doesNotExist"));
-    ErrorHierarchy errorDoesNotExists;
-    errorDoesNotExists.setError(obj.errorString(QJsonValue::Double, QJsonValue::Undefined));
-    errorDoesNotExists.prependMember("doesNotExist");
-    QCOMPARE(errorHierarchy, errorDoesNotExists);
-    errorHierarchy.clear();
-
-    QVERIFY(!obj.check<int>(&errorHierarchy, "bool"));
-    ErrorHierarchy errorWrongType;
-    errorWrongType.setError(obj.errorString(QJsonValue::Double, QJsonValue::Bool));
-    errorWrongType.prependMember("bool");
-    QCOMPARE(errorHierarchy, errorWrongType);
-    errorHierarchy.clear();
-
-    QVERIFY(obj.check<int>(&errorHierarchy, "integer"));
-    QVERIFY(errorHierarchy.isEmpty());
-    QVERIFY(obj.check<double>(&errorHierarchy, "double"));
-    QVERIFY(errorHierarchy.isEmpty());
-    QVERIFY(obj.check<bool>(&errorHierarchy, "bool"));
-    QVERIFY(errorHierarchy.isEmpty());
-    QVERIFY(obj.check<std::nullptr_t>(&errorHierarchy, "null"));
-    QVERIFY(errorHierarchy.isEmpty());
-    QVERIFY(obj.check<QString>(&errorHierarchy, "string"));
-    QVERIFY(errorHierarchy.isEmpty());
-}
-
 void tst_LanguageServerProtocol::documentUri_data()
 {
     QTest::addColumn<DocumentUri>("uri");
@@ -659,6 +540,62 @@ void tst_LanguageServerProtocol::documentUri()
     QCOMPARE(uri.toString(), string);
 }
 
-QTEST_MAIN(tst_LanguageServerProtocol)
+void tst_LanguageServerProtocol::range_data()
+{
+    QTest::addColumn<Range>("r1");
+    QTest::addColumn<Range>("r2");
+    QTest::addColumn<bool>("overlaps");
+    QTest::addColumn<bool>("r1Containsr2");
+    QTest::addColumn<bool>("r2Containsr1");
+
+    auto pos = [](int pos) { return Position(0, pos); };
+
+    QTest::newRow("both ranges empty")
+        << Range(pos(0), pos(0)) << Range(pos(0), pos(0)) << true << true << true;
+    QTest::newRow("equal ranges")
+        << Range(pos(0), pos(1)) << Range(pos(0), pos(1)) << true << true << true;
+    QTest::newRow("r1 before r2")
+        << Range(pos(0), pos(1)) << Range(pos(2), pos(3)) << false << false << false;
+    QTest::newRow("r1 adjacent before r2")
+        << Range(pos(0), pos(1)) << Range(pos(1), pos(2)) << false << false << false;
+    QTest::newRow("r1 starts before r2 overlapping")
+        << Range(pos(0), pos(2)) << Range(pos(1), pos(3)) << true << false << false;
+    QTest::newRow("empty r1 on r2 start")
+        << Range(pos(0), pos(0)) << Range(pos(0), pos(1)) << true << false << true;
+    QTest::newRow("r1 inside r2 equal start")
+        << Range(pos(0), pos(1)) << Range(pos(0), pos(2)) << true << false << true;
+    QTest::newRow("r1 inside r2 equal start")
+        << Range(pos(0), pos(1)) << Range(pos(0), pos(2)) << true << false << true;
+    QTest::newRow("empty r1 inside r2")
+        << Range(pos(1), pos(1)) << Range(pos(0), pos(2)) << true << false << true;
+    QTest::newRow("r1 completely inside r2")
+        << Range(pos(1), pos(2)) << Range(pos(0), pos(3)) << true << false << true;
+    QTest::newRow("r1 inside r2 equal end")
+        << Range(pos(1), pos(2)) << Range(pos(0), pos(2)) << true << false << true;
+    QTest::newRow("r1 ends after r2 overlapping")
+        << Range(pos(1), pos(3)) << Range(pos(0), pos(2)) << true << false << false;
+    QTest::newRow("empty r1 on r2 end")
+        << Range(pos(1), pos(1)) << Range(pos(0), pos(1)) << true << false << true;
+    QTest::newRow("r1 adjacent after r2")
+        << Range(pos(1), pos(2)) << Range(pos(0), pos(1)) << false << false << false;
+    QTest::newRow("r1 behind r2")
+        << Range(pos(2), pos(3)) << Range(pos(0), pos(1)) << false << false << false;
+}
+
+void tst_LanguageServerProtocol::range()
+{
+    QFETCH(Range, r1);
+    QFETCH(Range, r2);
+    QFETCH(bool, overlaps);
+    QFETCH(bool, r1Containsr2);
+    QFETCH(bool, r2Containsr1);
+
+    QVERIFY(r1.overlaps(r2) == r2.overlaps(r1));
+    QCOMPARE(r1.overlaps(r2), overlaps);
+    QCOMPARE(r1.contains(r2), r1Containsr2);
+    QCOMPARE(r2.contains(r1), r2Containsr1);
+}
+
+QTEST_GUILESS_MAIN(tst_LanguageServerProtocol)
 
 #include "tst_languageserverprotocol.moc"
